@@ -966,6 +966,7 @@ ipadb_fetch_principals_with_extra_filter(struct ipadb_context *ipactx,
     krb5_error_code kerr;
     char *src_filter = NULL, *esc_original_princ = NULL;
     int ret;
+    int len = 0;
 
     if (!ipactx->lcontext) {
         ret = ipadb_get_connection(ipactx);
@@ -983,6 +984,8 @@ ipadb_fetch_principals_with_extra_filter(struct ipadb_context *ipactx,
         goto done;
     }
 
+    len = strlen(esc_original_princ);
+
     /* Starting in DAL 8.0, aliases are always okay. */
 #ifdef KRB5_KDB_FLAG_ALIAS_OK
     if (!(flags & KRB5_KDB_FLAG_ALIAS_OK)) {
@@ -996,12 +999,24 @@ ipadb_fetch_principals_with_extra_filter(struct ipadb_context *ipactx,
     } else
 #endif
     {
+        /* In case we've got a principal name as '*' we have to
+         * follow RFC 4515 section 3 and reencode it using
+         * <valueencoding> rule from RFC 4511 section 4.1.6 but
+         * only to the part of the filter that does use assertion
+         * value. */
+        const char *asterisk = "%x2A";
+        char *assertion_value = esc_original_princ;
+
+        if ((len == 1) && (esc_original_princ[0] == '*')) {
+            assertion_value = asterisk;
+        }
+
         if (filter == NULL) {
             ret = asprintf(&src_filter, PRINC_TGS_SEARCH_FILTER,
-                           esc_original_princ, esc_original_princ);
+                           esc_original_princ, assertion_value);
         } else {
             ret = asprintf(&src_filter, PRINC_TGS_SEARCH_FILTER_EXTRA,
-                           esc_original_princ, esc_original_princ, filter);
+                           esc_original_princ, assertion_value, filter);
         }
     }
 
@@ -1039,6 +1054,7 @@ krb5_error_code ipadb_find_principal(krb5_context kcontext,
     struct berval **vals = NULL;
     int result;
     krb5_error_code ret;
+    size_t princ_len = 0;
 
     ipactx = ipadb_get_context(kcontext);
     if (!ipactx) {
@@ -1046,6 +1062,7 @@ krb5_error_code ipadb_find_principal(krb5_context kcontext,
         goto done;
     }
 
+    princ_len = strlen(*principal);
     for (le = ldap_first_entry(ipactx->lcontext, res); le != NULL;
          le = ldap_next_entry(ipactx->lcontext, le)) {
         vals = ldap_get_values_len(ipactx->lcontext, le, "krbprincipalname");
@@ -1057,7 +1074,8 @@ krb5_error_code ipadb_find_principal(krb5_context kcontext,
         for (int i = 0; vals[i]; i++) {
 #ifdef KRB5_KDB_FLAG_ALIAS_OK
             if ((flags & KRB5_KDB_FLAG_ALIAS_OK) == 0) {
-                found = strcmp(vals[i]->bv_val, *principal) == 0;
+                found = ((vals[i]->bv_len == princ_len) &&
+                         strncmp(vals[i]->bv_val, *principal, vals[i]->bv_len) == 0);
                 if (found)
                     break;
 
@@ -1069,7 +1087,7 @@ krb5_error_code ipadb_find_principal(krb5_context kcontext,
              * (ref_tgt_again in do_tgs_req.c), so use case-insensitive
              * comparison. */
             if (ulc_casecmp(vals[i]->bv_val, vals[i]->bv_len, *principal,
-                            strlen(*principal), NULL, NULL, &result) != 0) {
+                            princ_len, NULL, NULL, &result) != 0) {
                 ret = KRB5_KDB_INTERNAL_ERROR;
                 goto done;
             }
@@ -1080,19 +1098,23 @@ krb5_error_code ipadb_find_principal(krb5_context kcontext,
              * name/alias is returned even if krbCanonicalName is not
              * present. */
             free(*principal);
-            *principal = strdup(vals[i]->bv_val);
+            *principal = strndup(vals[i]->bv_val, vals[i]->bv_len);
             if (!*principal) {
                 ret = KRB5_KDB_INTERNAL_ERROR;
                 goto done;
             }
+            princ_len = strlen(*principal);
             found = true;
             break;
         }
-        if (!found)
+
+        ldap_value_free_len(vals);
+        vals = NULL;
+        if (!found) {
             continue;
+        }
 
         /* We need to check if this is the canonical name. */
-        ldap_value_free_len(vals);
         vals = ldap_get_values_len(ipactx->lcontext, le, "krbcanonicalname");
         if (vals == NULL)
             break;
@@ -1101,16 +1123,18 @@ krb5_error_code ipadb_find_principal(krb5_context kcontext,
         /* If aliases aren't accepted by the KDC, use case-sensitive
          * comparison. */
         if ((flags & KRB5_KDB_FLAG_ALIAS_OK) == 0) {
-            found = strcmp(vals[0]->bv_val, *principal) == 0;
+            found = ((vals[0]->bv_len == strlen(*principal)) &&
+                     strncmp(vals[0]->bv_val, *principal, vals[0]->bv_len) == 0);
             if (!found) {
                 ldap_value_free_len(vals);
+		vals = NULL;
                 continue;
             }
         }
 #endif
 
         free(*principal);
-        *principal = strdup(vals[0]->bv_val);
+        *principal = strndup(vals[0]->bv_val, vals[0]->bv_len);
         if (!*principal) {
             ret = KRB5_KDB_INTERNAL_ERROR;
             goto done;
@@ -1249,32 +1273,67 @@ done:
     return kerr;
 }
 
-/* TODO: handle case where main object and krbprincipal data are not
- * the same object but linked objects ?
- * (by way of krbprincipalaux being in a separate object from krbprincipal).
- * Currently we only support objcts with both objectclasses present at the
- * same time. */
+static krb5_boolean is_request_for_us(krb5_context kcontext,
+                                      krb5_principal local_tgs,
+                                      krb5_const_principal search_for)
+{
+    krb5_boolean for_us;
 
-krb5_error_code ipadb_get_principal(krb5_context kcontext,
-                                    krb5_const_principal search_for,
-                                    unsigned int flags,
-                                    krb5_db_entry **entry)
+    for_us = krb5_realm_compare(kcontext, local_tgs, search_for) ||
+             krb5_principal_compare_any_realm(kcontext,
+                                              local_tgs, search_for);
+    return for_us;
+}
+
+static krb5_error_code dbget_princ(krb5_context kcontext,
+                                   struct ipadb_context *ipactx,
+                                   krb5_const_principal search_for,
+                                   unsigned int flags,
+                                   krb5_db_entry **entry)
 {
-    struct ipadb_context *ipactx;
     krb5_error_code kerr;
     char *principal = NULL;
-    char *trusted_realm = NULL;
     LDAPMessage *res = NULL;
     LDAPMessage *lentry;
-    krb5_db_entry *kentry = NULL;
     uint32_t pol;
 
-    ipactx = ipadb_get_context(kcontext);
-    if (!ipactx) {
-        return KRB5_KDB_DBNOTINITED;
+
+    if ((flags & KRB5_KDB_FLAG_CLIENT_REFERRALS_ONLY) != 0 &&
+        (flags & KRB5_KDB_FLAG_CANONICALIZE) != 0) {
+
+        /* AS_REQ with canonicalization*/
+        krb5_principal norm_princ = NULL;
+
+        /* unparse the Kerberos principal without (our) outer realm. */
+        kerr = krb5_unparse_name_flags(kcontext, search_for,
+                                    KRB5_PRINCIPAL_UNPARSE_NO_REALM |
+                                    KRB5_PRINCIPAL_UNPARSE_DISPLAY,
+                                    &principal);
+        if (kerr != 0) {
+            goto done;
+        }
+
+        /* Re-parse the principal to normalize it. Innner realm becomes
+        * the realm if present. If no inner realm, our default realm
+        * will be used instead (as it was before). */
+        kerr = krb5_parse_name(kcontext, principal, &norm_princ);
+        if (kerr != 0) {
+            goto done;
+        }
+        /* Unparse without escaping '@' and '/' because we are going to use them
+        * in LDAP filters where escaping character '\' will be escaped and the
+        * result will never match. */
+        kerr = krb5_unparse_name_flags(kcontext, norm_princ,
+                                    KRB5_PRINCIPAL_UNPARSE_DISPLAY, &principal);
+        krb5_free_principal(kcontext, norm_princ);
+    } else {
+        /* Unparse without escaping '@' and '/' because we are going to use them
+        * in LDAP filters where escaping character '\' will be escaped and the
+        * result will never match. */
+        kerr = krb5_unparse_name_flags(kcontext, search_for,
+                                    KRB5_PRINCIPAL_UNPARSE_DISPLAY, &principal);
     }
 
-    kerr = krb5_unparse_name(kcontext, search_for, &principal);
     if (kerr != 0) {
         goto done;
     }
@@ -1286,95 +1345,7 @@ krb5_error_code ipadb_get_principal(krb5_context kcontext,
 
     kerr = ipadb_find_principal(kcontext, flags, res, &principal, &lentry);
     if (kerr != 0) {
-        if ((kerr == KRB5_KDB_NOENTRY) &&
-            ((flags & (KRB5_KDB_FLAG_CANONICALIZE |
-                       KRB5_KDB_FLAG_CLIENT_REFERRALS_ONLY)) != 0)) {
-
-            /* First check if we got enterprise principal which looks like
-             * username\@enterprise_realm@REALM */
-            char *realm;
-            krb5_data *upn;
-
-            upn = krb5_princ_component(kcontext, search_for,
-                                       krb5_princ_size(kcontext, search_for) - 1);
-
-            if (upn == NULL) {
-                kerr = KRB5_KDB_NOENTRY;
-                goto done;
-            }
-
-            realm = memrchr(upn->data, '@', upn->length);
-            if (realm == NULL) {
-                kerr = KRB5_KDB_NOENTRY;
-                goto done;
-            }
-
-            /* skip '@' and use part after '@' as an enterprise realm for comparison */
-            realm++;
-
-            /* check for our realm */
-            if (strncasecmp(ipactx->realm, realm,
-                            upn->length - (realm - upn->data)) == 0) {
-                /* it looks like it is ok to use malloc'ed strings as principal */
-                krb5_free_unparsed_name(kcontext, principal);
-                principal = strndup((const char *) upn->data, upn->length);
-                if (principal == NULL) {
-                    kerr = ENOMEM;
-                    goto done;
-                }
-
-                ldap_msgfree(res);
-                res = NULL;
-                kerr = ipadb_fetch_principals(ipactx, flags, principal, &res);
-                if (kerr != 0) {
-                    goto done;
-                }
-
-                kerr = ipadb_find_principal(kcontext, flags, res, &principal,
-                                            &lentry);
-                if (kerr != 0) {
-                    goto done;
-                }
-            } else {
-
-                kerr = ipadb_is_princ_from_trusted_realm(kcontext,
-                                                         realm,
-                                                         upn->length - (realm - upn->data),
-                                                         &trusted_realm);
-                if (kerr == KRB5_KDB_NOENTRY) {
-                    /* try to refresh trusted domain data and try again */
-                    kerr = ipadb_reinit_mspac(ipactx, false);
-                    if (kerr != 0) {
-                        kerr = KRB5_KDB_NOENTRY;
-                        goto done;
-                    }
-                    kerr = ipadb_is_princ_from_trusted_realm(kcontext, realm,
-                                              upn->length - (realm - upn->data),
-                                              &trusted_realm);
-                }
-                if (kerr == 0) {
-                    kentry = calloc(1, sizeof(krb5_db_entry));
-                    if (!kentry) {
-                        kerr = ENOMEM;
-                        goto done;
-                    }
-                    kerr = krb5_parse_name(kcontext, principal,
-                                           &kentry->princ);
-                    if (kerr != 0) {
-                        goto done;
-                    }
-
-                    kerr = krb5_set_principal_realm(kcontext, kentry->princ, trusted_realm);
-                    if (kerr != 0) {
-                        goto done;
-                    }
-                    *entry = kentry;
-                }
-                goto done;
-            }
-        } else {
-            goto done;
-        }
+        goto done;
     }
 
     kerr = ipadb_parse_ldap_entry(kcontext, principal, lentry, entry, &pol);
@@ -1390,15 +1361,187 @@ krb5_error_code ipadb_get_principal(krb5_context kcontext,
     }
 
 done:
-    free(trusted_realm);
-    if ((kerr != 0) && (kentry != NULL)) {
-        ipadb_free_principal(kcontext, kentry);
-    }
     ldap_msgfree(res);
     krb5_free_unparsed_name(kcontext, principal);
+
     return kerr;
 }
 
+static krb5_error_code dbget_alias(krb5_context kcontext,
+                                   struct ipadb_context *ipactx,
+                                   krb5_const_principal search_for,
+                                   unsigned int flags,
+                                   krb5_db_entry **entry)
+{
+    krb5_error_code kerr = 0;
+    char *principal = NULL;
+    krb5_principal norm_princ = NULL;
+    char *trusted_realm = NULL;
+    krb5_db_entry *kentry = NULL;
+    krb5_data *realm;
+
+    /* TODO: also support hostbased aliases */
+
+    /* Enterprise principal name type is for potential aliases or principals
+     * from trusted realms. The logic below only applies to this type */
+    if (krb5_princ_type(kcontext, search_for) != KRB5_NT_ENTERPRISE_PRINCIPAL) {
+        return KRB5_KDB_NOENTRY;
+    }
+
+    /* enterprise principal can only have single component in the name
+     * according to RFC6806 section 5. */
+    if (krb5_princ_size(kcontext, search_for) != 1) {
+        return KRB5_KDB_NOENTRY;
+    }
+
+    /* unparse the Kerberos principal without (our) outer realm. */
+    kerr = krb5_unparse_name_flags(kcontext, search_for,
+                                   KRB5_PRINCIPAL_UNPARSE_NO_REALM |
+                                   KRB5_PRINCIPAL_UNPARSE_DISPLAY,
+                                   &principal);
+    if (kerr != 0) {
+        goto done;
+    }
+
+    /* Re-parse the principal to normalize it. Innner realm becomes
+     * the realm if present. If no inner realm, our default realm
+     * will be used instead (as it was before). */
+    kerr = krb5_parse_name(kcontext, principal, &norm_princ);
+    if (kerr != 0) {
+        goto done;
+    }
+
+    if (krb5_realm_compare(kcontext, ipactx->local_tgs, norm_princ)) {
+        /* In realm alias, try to retrieve it and let the caller handle it. */
+        kerr = dbget_princ(kcontext, ipactx, norm_princ, flags, entry);
+        goto done;
+    }
+
+    /* The request is out of realm starting from here */
+
+    /*
+     * Per RFC6806 section 7 and 8, the canonicalize flag is required for
+     * both client and server referrals. But it is more useful to ignore it
+     * like Windows KDC does for client referrals.
+     */
+    if (((flags & KRB5_KDB_FLAG_CANONICALIZE) == 0) &&
+        ((flags & KRB5_KDB_FLAG_CLIENT_REFERRALS_ONLY) == 0)) {
+        kerr = KRB5_KDB_NOENTRY;
+        goto done;
+    }
+
+    /* Determine the trusted realm to refer to. We don't need the principal
+     * itself, only its realm */
+    realm = krb5_princ_realm(kcontext, norm_princ);
+    kerr = ipadb_is_princ_from_trusted_realm(kcontext,
+                                             realm->data,
+                                             realm->length,
+                                             &trusted_realm);
+    if (kerr == KRB5_KDB_NOENTRY) {
+        /* If no trusted realm found, refresh trusted domain data and try again
+         * because it might be a freshly added trust to AD */
+        kerr = ipadb_reinit_mspac(ipactx, false);
+        if (kerr != 0) {
+            kerr = KRB5_KDB_NOENTRY;
+            goto done;
+        }
+        kerr = ipadb_is_princ_from_trusted_realm(kcontext,
+                                                 realm->data,
+                                                 realm->length,
+                                                 &trusted_realm);
+    }
+
+    if (kerr != 0) {
+        goto done;
+    }
+
+    /* This is a known trusted realm. Issue a referral depending on whether this
+     * is client or server referral request */
+    if (flags & KRB5_KDB_FLAG_CLIENT_REFERRALS_ONLY) {
+        /* client referral out of realm, set next realm. */
+        kerr = krb5_set_principal_realm(kcontext, norm_princ, trusted_realm);
+        if (kerr != 0) {
+            goto done;
+        }
+        kentry = calloc(1, sizeof(krb5_db_entry));
+        if (!kentry) {
+            kerr = ENOMEM;
+            goto done;
+        }
+
+        kentry->princ = norm_princ;
+        norm_princ = NULL;
+        *entry = kentry;
+
+        goto done;
+    }
+
+    if (flags & KRB5_KDB_FLAG_INCLUDE_PAC) {
+        /* TGS request where KDC wants to generate PAC
+         * but the principal is out of our realm */
+        kerr = KRB5_KDB_NOENTRY;
+        goto done;
+    }
+
+    /* server referrals: lookup krbtgt/next_realm@our_realm */
+
+    krb5_free_principal(kcontext, norm_princ);
+    norm_princ = NULL;
+    kerr = krb5_build_principal_ext(kcontext, &norm_princ,
+                                    strlen(ipactx->realm),
+                                    ipactx->realm,
+                                    KRB5_TGS_NAME_SIZE,
+                                    KRB5_TGS_NAME,
+                                    strlen(trusted_realm),
+                                    trusted_realm, 0);
+    if (kerr != 0) {
+        goto done;
+    }
+
+    kerr = dbget_princ(kcontext, ipactx, norm_princ, flags, entry);
+
+done:
+    free(trusted_realm);
+    krb5_free_principal(kcontext, norm_princ);
+    krb5_free_unparsed_name(kcontext, principal);
+
+    return kerr;
+}
+
+/* TODO: handle case where main object and krbprincipal data are not
+ * the same object but linked objects ?
+ * (by way of krbprincipalaux being in a separate object from krbprincipal).
+ * Currently we only support objcts with both objectclasses present at the
+ * same time. */
+
+krb5_error_code ipadb_get_principal(krb5_context kcontext,
+                                    krb5_const_principal search_for,
+                                    unsigned int flags,
+                                    krb5_db_entry **entry)
+{
+    struct ipadb_context *ipactx;
+    krb5_error_code kerr;
+
+    *entry = NULL;
+
+    ipactx = ipadb_get_context(kcontext);
+    if (!ipactx) {
+        return KRB5_KDB_DBNOTINITED;
+    }
+
+    if (!is_request_for_us(kcontext, ipactx->local_tgs, search_for)) {
+        return KRB5_KDB_NOENTRY;
+    }
+
+    /* Lookup local names and aliases first. */
+    kerr = dbget_princ(kcontext, ipactx, search_for, flags, entry);
+    if (kerr != KRB5_KDB_NOENTRY) {
+        return kerr;
+    }
+
+    return dbget_alias(kcontext, ipactx, search_for, flags, entry);
+}
+
 void ipadb_free_principal_e_data(krb5_context kcontext, krb5_octet *e_data)
 {
     struct ipadb_e_data *ied;
@@ -2579,7 +2722,7 @@ krb5_error_code ipadb_delete_principal(krb5_context kcontext,
     char *canonicalized = NULL;
     LDAPMessage *res = NULL;
     LDAPMessage *lentry;
-    unsigned int flags;
+    unsigned int flags = 0;
 
     ipactx = ipadb_get_context(kcontext);
     if (!ipactx) {
