@@ -25,12 +25,14 @@ define([
         'dojo/_base/declare',
         'dojo/_base/lang',
         'dojo/dom-construct',
+        'dojo/topic',
         'dojo/on',
         'dojo/Stateful',
         'dojo/Evented',
         './_base/Singleton_registry',
         './_base/construct',
         './builder',
+        './config',
         './ipa',
         './jquery',
         './navigation',
@@ -43,8 +45,8 @@ define([
         './dialog',
         './field',
         './widget'
-       ], function(declare, lang, construct, on, Stateful, Evented,
-                   Singleton_registry, construct_utils, builder, IPA, $,
+    ], function(declare, lang, construct, topic, on, Stateful, Evented,
+                   Singleton_registry, construct_utils, builder, config, IPA, $,
                    navigation, phases, reg, rpc, su, text, ActionDropdownWidget) {
 
 /**
@@ -1380,6 +1382,8 @@ exp.facet_header = IPA.facet_header = function(spec) {
 
     var that = exp.simple_facet_header(spec);
 
+    that.adapter = builder.build('adapter', spec.adapter || 'adapter', { context: that });
+
     that.update_breadcrumb = function(pkey) {
 
         if (!that.breadcrumb) return;
@@ -1500,7 +1504,7 @@ exp.facet_header = IPA.facet_header = function(spec) {
      */
     that.load = function(data) {
         if (!data) return;
-        var result = data.result.result;
+        var result = that.adapter.get_record(data);
         if (!that.facet.disable_facet_tabs) {
             var pkey = that.facet.get_pkey();
 
@@ -1635,13 +1639,29 @@ exp.FacetGroupsWidget = declare([], {
         $('<a/>', {
             text: tab.tab_label,
             'class': 'tab-link',
-            href: "#" + navigation.create_hash(tab, {}),
+            href: "#" + navigation.create_hash(tab, {
+                pkeys: self.facet.get_pkeys()
+            }),
             name: tab.name
         }).appendTo(el);
 
         return el;
     },
 
+    hide_tab: function(tab_name) {
+        var tab = this.get_tab_el(tab_name);
+        if (tab) tab.css('display', 'none');
+    },
+
+    show_tab: function(tab_name) {
+        var tab = this.get_tab_el(tab_name);
+        if (tab) tab.css('display', '');
+    },
+
+    get_tab_el: function(tab_name) {
+        return this.tab_els[tab_name];
+    },
+
     on_click: function(facet) {
         if (this.facet.get_pkeys) {
             var pkeys = this.facet.get_pkeys();
@@ -1818,6 +1838,14 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
 
     var that = IPA.facet(spec, no_init);
 
+    /**
+     * Sets whether table on the facet will or will not show items with the
+     * same key.
+     *
+     * @property {boolean}
+     */
+    that.show_values_with_dup_key = spec.show_values_with_dup_key || false;
+
     /**
      * Names of additional row attributes which will be send to another facet
      * during navigation as URL parameters.
@@ -1844,6 +1872,14 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
      */
     that.search_all_entries = spec.search_all_entries;
 
+    /**
+     * Attribute from *_find command which will be used in batch *_show command
+     * which is called for each row.
+     *
+     * @property {String}
+     */
+    that.show_command_additional_attr = spec.show_command_additional_attr || null;
+
     /**
      * Member resolution(no_member: true ) in rpc request is skipped by default
      * to improve performance of getting data.
@@ -2050,11 +2086,13 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
      *
      * @protected
      * @param {Object} data RPC command data
-     * @return {ordered_map} record map
+     * @return {Object} ordered_maps with records and with pkeys. keys
+     *                              are composed from pkey and index.
      */
     that.get_records_map = function(data) {
 
         var records_map = $.ordered_map();
+        var pkeys_map = $.ordered_map();
 
         var result = data.result.result;
         var pkey_name = that.managed_entity.metadata.primary_key ||
@@ -2065,11 +2103,21 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
             var record = result[i];
             var pkey = adapter.load(record, pkey_name)[0];
             if (that.filter_records(records_map, pkey, record)) {
-                records_map.put(pkey, record);
+                // This solution allows to show tables where are the same
+                // primary keys. (i.e. {User|Service} Vaults)
+                var compound_pkey = pkey;
+                if (that.show_values_with_dup_key) {
+                    compound_pkey = pkey + i;
+                }
+                records_map.put(compound_pkey, record);
+                pkeys_map.put(compound_pkey, pkey);
             }
         }
 
-        return records_map;
+        return {
+            records_map: records_map,
+            pkeys_map: pkeys_map
+        };
     };
 
     /**
@@ -2086,7 +2134,9 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
     that.load_page = function(data) {
 
         // get primary keys (and the complete records if search_all_entries is true)
-        var records_map = that.get_records_map(data);
+        var records = that.get_records_map(data);
+        var records_map = records.records_map;
+        var pkeys_map = records.pkeys_map;
 
         var total = records_map.length;
         that.table.total_pages = total ? Math.ceil(total / that.table.page_length) : 1;
@@ -2122,37 +2172,38 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
 
         // sort map based on primary keys
         if (that.sort_enabled) {
-            records_map = records_map.sort();
+            pkeys_map = pkeys_map.sort();
         }
 
         // trim map leaving the entries visible in the current page only
-        records_map = records_map.slice(start-1, end);
+        pkeys_map = pkeys_map.slice(start-1, end);
 
         var columns = that.table.columns.values;
-        if (columns.length == 1) { // show primary keys only
-            that.load_records(records_map.values);
-            return;
-        }
-
-        if (that.search_all_entries) {
-            // map contains the primary keys and the complete records
-            that.load_records(records_map.values);
+        if (columns.length == 1 || that.search_all_entries) {
+            // All needed pkeys/objects are already fetched from server,
+            // so we just filter and show them.
+            that.load_records(pkeys_map.keys.map(function(x) {
+                return records_map.get(x);
+            }));
             return;
         }
 
         // get the complete records
         that.get_records(
-            records_map.keys,
+            records_map,
+            pkeys_map,
             function(data, text_status, xhr) {
                 var results = data.result.results;
-                for (var i=0; i<records_map.length; i++) {
-                    var pkey = records_map.keys[i];
+                var show_records_map = $.ordered_map();
+                for (var i=0; i<pkeys_map.length; i++) {
+                    var pkey = pkeys_map.keys[i];
                     var record = records_map.get(pkey);
                     // merge the record obtained from the refresh()
                     // with the record obtained from get_records()
                     $.extend(record, results[i].result);
+                    show_records_map.put(pkey, record);
                 }
-                that.load_records(records_map.values);
+                that.load_records(show_records_map.values);
             },
             function(xhr, text_status, error_thrown) {
                 that.load_records([]);
@@ -2225,7 +2276,9 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
      * @param {Function} on_success command success handler
      * @param {Function} on_failure command error handler
      */
-    that.create_get_records_command = function(pkeys, on_success, on_error) {
+    that.create_get_records_command = function(records, pkeys_list, on_success, on_error) {
+
+        var pkeys = pkeys_list.keys;
 
         var batch = rpc.batch_command({
             name: that.get_records_command_name(),
@@ -2235,13 +2288,18 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
 
         for (var i=0; i<pkeys.length; i++) {
             var pkey = pkeys[i];
+            var call_pkey = pkeys_list.get(pkey);
 
             var command = rpc.command({
                 entity: that.table.entity.name,
                 method: 'show',
-                args: [pkey]
+                args: [call_pkey]
             });
 
+            if (that.show_command_additional_attr) {
+                that.extend_get_records_command(command, records, pkey);
+            }
+
             if (!that.always_request_members && that.table.entity.has_members()) {
                 command.set_options({no_members: true});
             }
@@ -2252,17 +2310,35 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
         return batch;
     };
 
+
+    /**
+     * This allows to use pagination in situations when for loading whole search
+     * page you need *_show command with
+     *
+     */
+    that.extend_get_records_command = function(command, records, pkey) {
+        var record = records.get(pkey);
+        var item = record[that.show_command_additional_attr];
+        if (item) {
+            var temp_option = {};
+            temp_option[that.show_command_additional_attr] = item;
+            command.set_options(temp_option);
+        }
+    };
+
+
+
     /**
      * Execute command for obtaining complete records
      *
      * @protected
-     * @param {Array.<string>} pkeys primary keys
+     * @param records_map of all records
      * @param {Function} on_success command success handler
      * @param {Function} on_failure command error handler
      */
-    that.get_records = function(pkeys, on_success, on_error) {
+    that.get_records = function(records, pkeys, on_success, on_error) {
 
-        var batch = that.create_get_records_command(pkeys, on_success, on_error);
+        var batch = that.create_get_records_command(records, pkeys, on_success, on_error);
 
         batch.execute();
     };
@@ -2309,16 +2385,15 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
     };
 
     /**
+     * Returns path to an entity details page
      *
-     * Method which will be called after clicking on pkey in table.
+     * It can be overridden by child classes.
      *
-     * It can be overridden by child classes for changing afterclick behavior.
-     *
-     * @param {String} value automatically filed by clicking
-     * @param {entity.entity} table entity
-     * @return {boolean} false
+     * @param {String} value - table column value
+     * @param {entity.entity} entity
+     * @return {Array} path to given entity
      */
-    that.on_column_link_click = function(value, entity) {
+    that.get_column_entity_path = function(value, entity) {
         var pkeys = [value];
         var args;
 
@@ -2334,8 +2409,9 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
             pkeys.push(value);
         }
 
-        navigation.show_entity(entity.name, that.details_facet_name, pkeys, args);
-        return false;
+        return navigation.get_entity_path(
+            entity.name, that.details_facet_name, pkeys, args
+        );
     };
 
     /**
@@ -2359,6 +2435,14 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
             selectable: that.selectable && !that.read_only
         });
 
+        topic.subscribe("change-pagination", function() {
+            that.table.page_length = config.get('table_page_size');
+
+            that.set_expired_flag();
+
+            if (that.is_shown) that.refresh();
+        });
+
         var columns = that.columns.values;
         for (var i=0; i<columns.length; i++) {
             var column = columns[i];
@@ -2371,8 +2455,8 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
             }
 
             if (column.link && column.primary_key) {
-                column.link_handler = function(value) {
-                    return that.on_column_link_click(value, entity);
+                column.get_entity_path = function(value) {
+                    return that.get_column_entity_path(value, entity);
                 };
             }
 
@@ -2421,9 +2505,16 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
         }
     };
 
+    that.fetch_records = function() {
+        if (!that.table) return null;
+
+        return that.table.records;
+    };
+
     if (!no_init) that.init_table_columns();
 
     that.table_facet_create_get_records_command = that.create_get_records_command;
+    that.table_facet_load_records = that.load_records;
 
     return that;
 };
@@ -3096,6 +3187,13 @@ exp.state_evaluator = IPA.state_evaluator = function(spec) {
      */
     that.first_pass = true;
 
+    /**
+     * Adapter which selects values from record on load.
+     *
+     * @property {IPA.Adapter}
+     */
+    that.adapter = builder.build('adapter', spec.adapter || 'adapter', { context: that });
+
     /**
      * Init the evaluator
      *
