@@ -326,6 +326,14 @@ class APINameSpace(Mapping):
         self.__enumerate()
         return iter(self.__plugins)
 
+    def __dir__(self):
+        # include plugins for readline tab completion and in dir()
+        self.__enumerate()
+        names = super().__dir__()
+        names.extend(p.name for p in self)
+        names.sort()
+        return names
+
     def get_plugin(self, key):
         self.__enumerate()
         return self.__plugins_by_key[key]
@@ -589,13 +597,18 @@ class API(ReadOnly):
             assert type(options.env) is list
             for item in options.env:
                 try:
-                    (key, value) = item.split('=', 1)
+                    values = item.split('=', 1)
                 except ValueError:
                     # FIXME: this should raise an IPA exception with an
                     # error code.
                     # --Jason, 2008-10-31
                     pass
-                overrides[str(key.strip())] = value.strip()
+                if len(values) == 2:
+                    (key, value) = values
+                    overrides[str(key.strip())] = value.strip()
+                else:
+                    raise errors.OptionError(_('Unable to parse option {item}'
+                                               .format(item=item)))
         for key in ('conf', 'debug', 'verbose', 'prompt_all', 'interactive',
             'fallback', 'delegate'):
             value = getattr(options, key, None)
