@@ -42,7 +42,7 @@ import struct
 import subprocess
 
 import netaddr
-from dns import resolver, rdatatype
+from dns import rdatatype
 from dns.exception import DNSException
 from dns.resolver import NXDOMAIN
 from netaddr.core import AddrFormatError
@@ -60,13 +60,19 @@ from ipalib.constants import (
     TLS_VERSIONS, TLS_VERSION_MINIMAL, TLS_VERSION_MAXIMAL,
     TLS_VERSION_DEFAULT_MIN, TLS_VERSION_DEFAULT_MAX,
 )
+# pylint: disable=ipa-forbidden-import
+from ipalib.facts import is_ipa_client_configured
 from ipalib.text import _
 from ipaplatform.constants import constants
 from ipaplatform.paths import paths
 from ipapython.ssh import SSHPublicKey
 from ipapython.dn import DN, RDN
-from ipapython.dnsutil import DNSName
-from ipapython.dnsutil import resolve_ip_addresses
+from ipapython.dnsutil import (
+    DNSName,
+    DNSResolver,
+    resolve,
+    resolve_ip_addresses,
+)
 from ipapython.admintool import ScriptError
 
 if sys.version_info >= (3, 2):
@@ -115,13 +121,13 @@ def has_soa_or_ns_record(domain):
     Returns True or False.
     """
     try:
-        resolver.query(domain, rdatatype.SOA)
+        resolve(domain, rdatatype.SOA)
         soa_record_found = True
     except DNSException:
         soa_record_found = False
 
     try:
-        resolver.query(domain, rdatatype.NS)
+        resolve(domain, rdatatype.NS)
         ns_record_found = True
     except DNSException:
         ns_record_found = False
@@ -795,7 +801,7 @@ def _resolve_record(owner, rtype, nameserver_ip=None, edns0=False,
     assert isinstance(nameserver_ip, str) or nameserver_ip is None
     assert isinstance(rtype, str)
 
-    res = dns.resolver.Resolver()
+    res = DNSResolver()
     if nameserver_ip:
         res.nameservers = [nameserver_ip]
     res.lifetime = timeout
@@ -813,7 +819,7 @@ def _resolve_record(owner, rtype, nameserver_ip=None, edns0=False,
     elif edns0:
         res.use_edns(0, 0, 4096)
 
-    return res.query(owner, rtype)
+    return res.resolve(owner, rtype)
 
 
 def _validate_edns0_forwarder(owner, rtype, ip_addr, timeout=10):
@@ -983,7 +989,7 @@ def detect_dns_zone_realm_type(api, domain):
     kerberos_record_name = kerberos_prefix + domain_suffix
 
     try:
-        result = resolver.query(kerberos_record_name, rdatatype.TXT)
+        result = resolve(kerberos_record_name, rdatatype.TXT)
         answer = result.response.answer
 
         # IPA domain will have only one _kerberos TXT record
@@ -1010,7 +1016,7 @@ def detect_dns_zone_realm_type(api, domain):
 
     try:
         # The presence of this record is enough, return foreign in such case
-        resolver.query(ad_specific_record_name, rdatatype.SRV)
+        resolve(ad_specific_record_name, rdatatype.SRV)
     except DNSException:
         # If we could not detect type with certainty, return unknown
         return 'unknown'
@@ -1027,6 +1033,7 @@ class classproperty:
     __slots__ = ('__doc__', 'fget')
 
     def __init__(self, fget=None, doc=None):
+        assert isinstance(fget, classmethod)
         if doc is None and fget is not None:
             doc = fget.__doc__
 
@@ -1049,6 +1056,17 @@ class classproperty:
         return self
 
 
+class classobjectproperty(classproperty):
+    # A class property that also passes the object to the getter
+    # obj is None for class objects and 'self' for instance objects.
+    __slots__ = ('__doc__',)
+
+    def __get__(self, obj, obj_type):
+        if self.fget is not None:
+            return self.fget.__get__(obj, obj_type)(obj)
+        raise AttributeError("unreadable attribute")
+
+
 def normalize_hostname(hostname):
     """Use common fqdn form without the trailing dot"""
     if hostname.endswith(u'.'):
@@ -1162,6 +1180,12 @@ def check_client_configuration(env=None):
     """
     Check if IPA client is configured on the system.
 
+    This is a convenience wrapper that also supports using
+    a custom configuration via IPA_CONFDIR.
+
+    Raises a ScriptError exception if the client is not
+    configured.
+
     Hardcode return code to avoid recursive imports
     """
     CLIENT_NOT_CONFIGURED = 2
@@ -1175,16 +1199,10 @@ def check_client_configuration(env=None):
                 f'{env.confdir} is missing {env.conf_default})',
                 CLIENT_NOT_CONFIGURED
             )
-    elif (
-            os.path.isfile(paths.IPA_DEFAULT_CONF) and
-            os.path.isfile(
-                os.path.join(paths.IPA_CLIENT_SYSRESTORE, 'sysrestore.state')
-            )
-    ):
-        # standard installation, check for config and client sysrestore state
+
+    if is_ipa_client_configured():
         return True
     else:
-        # client not configured
         raise ScriptError(
             'IPA client is not configured on this system',
             CLIENT_NOT_CONFIGURED
