@@ -43,9 +43,6 @@ import locale
 import collections
 import urllib
 
-from dns import resolver, reversename
-from dns.exception import DNSException
-
 import six
 from six.moves import input
 
@@ -538,7 +535,7 @@ def run(args, stdin=None, raiseonerr=True, nolog=(), env=None,
         logger.debug('Process interrupted')
         p.wait()
         raise
-    except:
+    except BaseException:
         logger.debug('Process execution failed')
         raise
     finally:
@@ -701,7 +698,7 @@ class CIDict(dict):
     if six.PY2:
         def has_key(self, key):
             # pylint: disable=no-member
-            return super(CIDict, self).has_key(key.lower())
+            return super(CIDict, self).has_key(key.lower())  # noqa
             # pylint: enable=no-member
 
     def get(self, key, failobj=None):
@@ -1112,22 +1109,6 @@ def check_port_bindable(port, socket_type=socket.SOCK_STREAM):
         s.close()
 
 
-def reverse_record_exists(ip_address):
-    """
-    Checks if IP address have some reverse record somewhere.
-    Does not care where it points.
-
-    Returns True/False
-    """
-    reverse = reversename.from_address(str(ip_address))
-    try:
-        resolver.query(reverse, "PTR")
-    except DNSException:
-        # really don't care what exception, PTR is simply unresolvable
-        return False
-    return True
-
-
 def config_replace_variables(filepath, replacevars=dict(), appendvars=dict(),
                              removevars=None):
     """
@@ -1580,7 +1561,7 @@ class APIVersion(tuple):
         return "<APIVersion('{}.{}')>".format(*self)
 
     def __getnewargs__(self):
-        return str(self)
+        return (str(self),)
 
     @property
     def major(self):
