@@ -28,6 +28,9 @@ import os
 import shutil
 import traceback
 import dbus
+import re
+import pwd
+import lxml.etree
 
 from configparser import DEFAULTSECT, ConfigParser, RawConfigParser
 
@@ -42,6 +45,7 @@ from ipalib.constants import CA_DBUS_TIMEOUT, IPA_CA_RECORD, RENEWAL_CA_NAME
 from ipaplatform import services
 from ipaplatform.constants import constants
 from ipaplatform.paths import paths
+from ipaplatform.tasks import tasks
 from ipapython import directivesetter
 from ipapython import ipaldap
 from ipapython import ipautil
@@ -52,10 +56,12 @@ from ipaserver.install import replication
 from ipaserver.install.installutils import stopped_service
 
 
-
 logger = logging.getLogger(__name__)
 
 
+INTERNAL_TOKEN = "internal"
+
+
 def get_security_domain():
     """
     Get the security domain from the REST interface on the local Dogtag CA
@@ -64,7 +70,8 @@ def get_security_domain():
     connection = PKIConnection(
         protocol='https',
         hostname=api.env.ca_host,
-        port='8443'
+        port='8443',
+        cert_paths=paths.IPA_CA_CRT
     )
     domain_client = pki.system.SecurityDomainClient(connection)
     info = domain_client.get_security_domain_info()
@@ -100,9 +107,8 @@ class DogtagInstance(service.Service):
     # dict.  The profile MUST be specified.
     tracking_reqs = dict()
 
-    # token for CA and subsystem certificates. For now, only internal token
-    # is supported.
-    token_name = "internal"
+    # HSM state is shared between CA and KRA
+    hsm_sstore = 'pki_hsm'
 
     # override token for specific nicknames
     token_names = dict()
@@ -157,6 +163,7 @@ class DogtagInstance(service.Service):
         self.pki_config_override = None
         self.ca_subject = None
         self.subject_base = None
+        self.ajp_secret = None
 
     def is_installed(self):
         """
@@ -176,7 +183,8 @@ class DogtagInstance(service.Service):
         subsystem = self.subsystem
         args = [paths.PKISPAWN,
                 "-s", subsystem,
-                "-f", cfg_file]
+                "-f", cfg_file,
+                "--debug"]
 
         with open(cfg_file) as f:
             logger.debug(
@@ -218,7 +226,7 @@ class DogtagInstance(service.Service):
         Enable client auth connection to the internal db.
         """
         sub_system_nickname = "subsystemCert cert-pki-ca"
-        if self.token_name != "internal":
+        if self.token_name != INTERNAL_TOKEN:
             # TODO: Dogtag 10.6.9 does not like "internal" prefix.
             sub_system_nickname = '{}:{}'.format(
                 self.token_name, sub_system_nickname
@@ -274,6 +282,86 @@ class DogtagInstance(service.Service):
             logger.critical("failed to uninstall %s instance %s",
                             self.subsystem, e)
 
+    def __is_newer_tomcat_version(self, default=None):
+        try:
+            result = ipautil.run([paths.BIN_TOMCAT, "version"],
+                                 capture_output=True)
+            sn = re.search(
+                r'Server number:\s+([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)',
+                result.output)
+            if sn is None:
+                logger.info("tomcat version cannot be parsed, "
+                            "default to pre-%s", default)
+                return False
+            v = tasks.parse_ipa_version(sn.group(1))
+            if v >= tasks.parse_ipa_version(default):
+                return True
+        except ipautil.CalledProcessError as e:
+            logger.info(
+                "failed to discover tomcat version, "
+                "default to pre-%s, error: %s",
+                default, str(e))
+        return False
+
+    def secure_ajp_connector(self):
+        """ Update AJP connector to use a password protection  """
+
+        server_xml = lxml.etree.parse(paths.PKI_TOMCAT_SERVER_XML)
+        doc = server_xml.getroot()
+
+        # no AJP connector means no need to update anything
+        connectors = doc.xpath('//Connector[@protocol="AJP/1.3"]')
+        if len(connectors) == 0:
+            return
+
+        # AJP protocol is at version 1.3. Assume there is only one as
+        # Dogtag only provisions one.
+        connector = connectors[0]
+
+        # Detect tomcat version and choose the right option name
+        # pre-9.0.31.0 uses 'requiredSecret'
+        # 9.0.31.0 or later uses 'secret'
+        secretattr = 'requiredSecret'
+        oldattr = 'requiredSecret'
+        if self.__is_newer_tomcat_version('9.0.31.0'):
+            secretattr = 'secret'
+
+        rewrite = True
+        if secretattr in connector.attrib:
+            # secret is already in place
+            # Perhaps, we need to synchronize it with Apache configuration
+            self.ajp_secret = connector.attrib[secretattr]
+            rewrite = False
+        else:
+            if oldattr in connector.attrib:
+                # Sufficiently new Dogtag versions (10.9.0-a2) handle the
+                # upgrade for us; we need only to ensure that we're not both
+                # attempting to upgrade server.xml at the same time.
+                # Hopefully this is guaranteed for us.
+                self.ajp_secret = connector.attrib[oldattr]
+                connector.attrib[secretattr] = self.ajp_secret
+                del connector.attrib[oldattr]
+            else:
+                # Generate password, don't use special chars to not break XML.
+                #
+                # If we hit this case, pkispawn was run on an older Dogtag
+                # version and we're stuck migrating, choosing a password
+                # ourselves. Dogtag can't generate one randomly because a
+                # Dogtag administrator might've configured AJP and might
+                # not be using IPA.
+                #
+                # Newer Dogtag versions will generate a random password
+                # during pkispawn.
+                self.ajp_secret = ipautil.ipa_generate_password(special=None)
+                connector.attrib[secretattr] = self.ajp_secret
+
+        if rewrite:
+            pent = pwd.getpwnam(constants.PKI_USER)
+            with open(paths.PKI_TOMCAT_SERVER_XML, "wb") as fd:
+                server_xml.write(fd, pretty_print=True, encoding="utf-8")
+                os.fchmod(fd.fileno(), 0o660)
+                os.fchown(fd.fileno(), pent.pw_uid, pent.pw_gid)
+
     def http_proxy(self):
         """ Update the http proxy file  """
         template_filename = (
@@ -283,13 +371,16 @@ class DogtagInstance(service.Service):
             DOGTAG_PORT=8009,
             CLONE='' if self.clone else '#',
             FQDN=self.fqdn,
+            DOGTAG_AJP_SECRET='',
         )
+        if self.ajp_secret:
+            sub_dict['DOGTAG_AJP_SECRET'] = "secret={}".format(self.ajp_secret)
         template = ipautil.template_file(template_filename, sub_dict)
         with open(paths.HTTPD_IPA_PKI_PROXY_CONF, "w") as fd:
             fd.write(template)
-            os.fchmod(fd.fileno(), 0o644)
+            os.fchmod(fd.fileno(), 0o640)
 
-    def configure_certmonger_renewal(self):
+    def configure_certmonger_renewal_helpers(self):
         """
         Create a new CA type for certmonger that will retrieve updated
         certificates from the dogtag master server.
@@ -305,7 +396,11 @@ class DogtagInstance(service.Service):
         obj = bus.get_object('org.fedorahosted.certmonger',
                              '/org/fedorahosted/certmonger')
         iface = dbus.Interface(obj, 'org.fedorahosted.certmonger')
-        for suffix, args in [('', ''), ('-reuse', ' --reuse-existing')]:
+        for suffix, args in [
+            ('', ''),
+            ('-reuse', ' --reuse-existing'),
+            ('-selfsigned', ' --force-self-signed'),
+        ]:
             name = RENEWAL_CA_NAME + suffix
             path = iface.find_ca_by_nickname(name)
             if not path:
@@ -317,7 +412,7 @@ class DogtagInstance(service.Service):
                     # Give dogtag extra time to generate cert
                     timeout=CA_DBUS_TIMEOUT)
 
-    def __get_pin(self, token_name="internal"):
+    def __get_pin(self, token_name=INTERNAL_TOKEN):
         try:
             return certmonger.get_pin(token_name)
         except IOError as e:
@@ -347,7 +442,10 @@ class DogtagInstance(service.Service):
                     "certmonger failed to start tracking certificate: %s", e)
 
     def stop_tracking_certificates(self, stop_certmonger=True):
-        """Stop tracking our certificates. Called on uninstall.
+        """
+        Stop tracking our certificates. Called on uninstall.  Also called
+        during upgrade to fix discrepancies.
+
         """
         logger.debug(
             "Configuring certmonger to stop tracking system certificates "
@@ -481,7 +579,8 @@ class DogtagInstance(service.Service):
             self.master_host
         )
         logger.debug(
-            "Waiting for %s to appear on %s", self.admin_dn, master_conn
+            "Waiting %s seconds for %s to appear on %s",
+            api.env.replication_wait_timeout, self.admin_dn, master_conn
         )
         deadline = time.time() + api.env.replication_wait_timeout
         while time.time() < deadline:
@@ -498,6 +597,9 @@ class DogtagInstance(service.Service):
             logger.error(
                 "Unable to log in as %s on %s", self.admin_dn, master_conn
             )
+            logger.info(
+                "[hint] tune with replication_wait_timeout"
+            )
             raise errors.NotFound(
                 reason="{} did not replicate to {}".format(
                     self.admin_dn, master_conn
@@ -587,6 +689,34 @@ class DogtagInstance(service.Service):
         )
         sysupgrade.set_upgrade_state('dogtag', state_name, True)
 
+    def set_hsm_state(self, config):
+        section_name = self.subsystem.upper()
+        assert section_name == 'CA'
+        if config.getboolean(section_name, 'pki_hsm_enable', fallback=False):
+            enable = True
+            token_name = config.get(section_name, 'pki_token_name')
+        else:
+            enable = False
+            token_name = INTERNAL_TOKEN
+        self.sstore.backup_state(self.hsm_sstore, "enabled", enable)
+        self.sstore.backup_state(self.hsm_sstore, "token_name", token_name)
+
+    def restore_hsm_state(self):
+        return (
+            self.sstore.restore_state(self.hsm_sstore, "enabled"),
+            self.sstore.restore_state(self.hsm_sstore, "token_name"),
+        )
+
+    @property
+    def hsm_enabled(self):
+        """Is HSM support enabled?"""
+        return self.sstore.get_state(self.hsm_sstore, "enabled")
+
+    @property
+    def token_name(self):
+        """HSM token name"""
+        return self.sstore.get_state(self.hsm_sstore, "token_name")
+
     def _configure_clone(self, subsystem_config, security_domain_hostname,
                          clone_pkcs12_path):
         subsystem_config.update(
@@ -695,6 +825,7 @@ class PKIIniLoader:
         self.defaults = dict(
             # pretty much static
             ipa_ca_pem_file=paths.IPA_CA_CRT,
+            pki_configuration_path=paths.PKI_CONFIGURATION,
             # variable
             ipa_ca_subject=ca_subject,
             ipa_subject_base=subject_base,
@@ -712,7 +843,9 @@ class PKIIniLoader:
             pki_subsystem_type=subsystem.lower(),
             home_dir=os.path.expanduser("~"),
             # for softhsm2 testing
-            softhsm2_so=paths.LIBSOFTHSM2_SO
+            softhsm2_so=paths.LIBSOFTHSM2_SO,
+            # Configure a more secure AJP password by default
+            ipa_ajp_secret=ipautil.ipa_generate_password(special=None)
         )
 
     @classmethod
