@@ -18,9 +18,9 @@
 #
 
 from __future__ import absolute_import
+from __future__ import division
 from __future__ import print_function
 
-import errno
 import logging
 import socket
 import getpass
@@ -34,32 +34,28 @@ import tempfile
 import shutil
 import traceback
 import textwrap
+import warnings
 from contextlib import contextmanager
+from configparser import ConfigParser as SafeConfigParser
+from configparser import NoOptionError
 
-from dns import resolver, rdatatype
+from dns import rdatatype
 from dns.exception import DNSException
 import ldap
-import ldapurl
 import six
-# pylint: disable=import-error
-if six.PY3:
-    # The SafeConfigParser class has been renamed to ConfigParser in Py3
-    from configparser import ConfigParser as SafeConfigParser
-else:
-    from ConfigParser import SafeConfigParser
-from six.moves.configparser import NoOptionError
-# pylint: enable=import-error
 
-from ipalib.install import sysrestore
+from ipalib import facts
 from ipalib.install.kinit import kinit_password
 import ipaplatform
-from ipapython import ipautil, admintool, version
+from ipapython import ipautil, admintool, version, ipaldap
 from ipapython.admintool import ScriptError, SERVER_NOT_CONFIGURED  # noqa: E402
 from ipapython.certdb import EXTERNAL_CA_TRUST_FLAGS
-from ipapython.ipaldap import DIRMAN_DN, LDAPClient
+from ipalib.constants import MAXHOSTNAMELEN
 from ipalib.util import validate_hostname
 from ipalib import api, errors, x509
+from ipalib.install import dnsforwarders
 from ipapython.dn import DN
+from ipapython.dnsutil import resolve
 from ipaserver.install import certs, service, sysupgrade
 from ipaplatform import services
 from ipaplatform.paths import paths
@@ -70,10 +66,6 @@ if six.PY3:
 
 logger = logging.getLogger(__name__)
 
-# Used to determine install status
-IPA_MODULES = [
-    'httpd', 'kadmin', 'dirsrv', 'pki-tomcatd', 'install', 'krb5kdc', 'named']
-
 
 class BadHostError(Exception):
     pass
@@ -111,7 +103,7 @@ class UpgradeMissingVersionError(UpgradeVersionError):
     pass
 
 
-class ReplicaConfig(object):
+class ReplicaConfig:
     def __init__(self, top_dir=None):
         self.realm_name = ""
         self.domain_name = ""
@@ -161,7 +153,7 @@ def verify_fqdn(host_name, no_host_dns=False, local_hostname=True):
 
     try:
         # make sure that the host name meets the requirements in ipalib
-        validate_hostname(host_name)
+        validate_hostname(host_name, maxlen=MAXHOSTNAMELEN)
     except ValueError as e:
         raise BadHostError("Invalid hostname '%s', %s" % (host_name, unicode(e)))
 
@@ -198,7 +190,7 @@ def verify_fqdn(host_name, no_host_dns=False, local_hostname=True):
     # Verify this is NOT a CNAME
     try:
         logger.debug('Check if %s is not a CNAME', host_name)
-        resolver.query(host_name, rdatatype.CNAME)
+        resolve(host_name, rdatatype.CNAME)
         raise HostReverseLookupError("The IPA Server Hostname cannot be a CNAME, only A and AAAA names are allowed.")
     except DNSException:
         pass
@@ -295,13 +287,28 @@ def read_ip_addresses():
 def read_dns_forwarders():
     addrs = []
     if ipautil.user_input("Do you want to configure DNS forwarders?", True):
-        print("Following DNS servers are configured in /etc/resolv.conf: %s" %
-                ", ".join(resolver.get_default_resolver().nameservers))
+        if dnsforwarders.detect_resolve1_resolv_conf():
+            servers = [
+                str(s) for s in dnsforwarders.get_resolve1_nameservers()
+            ]
+            print(
+                "The following DNS servers are configured in "
+                "systemd-resolved: %s" % ", ".join(servers)
+            )
+        else:
+            servers = [
+                str(s) for s in dnsforwarders.get_dnspython_nameservers()
+            ]
+            print(
+                "Following DNS servers are configured in /etc/resolv.conf: "
+                "%s" % ", ".join(servers)
+            )
+
         if ipautil.user_input("Do you want to configure these servers as DNS "
                 "forwarders?", True):
-            addrs = resolver.default_resolver.nameservers[:]
-            print("All DNS servers from /etc/resolv.conf were added. You can "
-                  "enter additional addresses now:")
+            addrs = servers[:]
+            print("All detected DNS servers were added. You can enter "
+                  "additional addresses now:")
         while True:
             ip = ipautil.user_input("Enter an IP address for a DNS forwarder, "
                                     "or press Enter to skip", allow_empty=True)
@@ -314,11 +321,18 @@ def read_dns_forwarders():
                 print("DNS forwarder %s not added." % ip)
                 continue
 
+            if ip_parsed.is_loopback():
+                print("Error: %s is a loopback address" % ip)
+                print("DNS forwarder %s not added." % ip)
+                continue
+
             print("DNS forwarder %s added. You may add another." % ip)
             addrs.append(str(ip_parsed))
 
     if not addrs:
         print("No DNS forwarders configured")
+    else:
+        print("DNS forwarders: %s" % ", ".join(addrs))
 
     return addrs
 
@@ -344,9 +358,9 @@ def validate_dm_password_ldap(password):
     Validate DM password by attempting to connect to LDAP. api.env has to
     contain valid ldap_uri.
     """
-    client = LDAPClient(api.env.ldap_uri, cacert=paths.IPA_CA_CRT)
+    client = ipaldap.LDAPClient(api.env.ldap_uri, cacert=paths.IPA_CA_CRT)
     try:
-        client.simple_bind(DIRMAN_DN, password)
+        client.simple_bind(ipaldap.DIRMAN_DN, password)
     except errors.ACIError:
         raise ValueError("Invalid Directory Manager password")
     else:
@@ -676,8 +690,7 @@ def check_server_configuration():
     Most convenient use case for the function is in install tools that require
     configured IPA for its function.
     """
-    server_fstore = sysrestore.FileStore(paths.SYSRESTORE)
-    if not server_fstore.has_files():
+    if not facts.is_ipa_configured():
         raise ScriptError("IPA is not configured on this system.",
                           rval=SERVER_NOT_CONFIGURED)
 
@@ -685,49 +698,36 @@ def check_server_configuration():
 def remove_file(filename):
     """Remove a file and log any exceptions raised.
     """
-    try:
-        os.unlink(filename)
-    except Exception as e:
-        # ignore missing file
-        if getattr(e, 'errno', None) != errno.ENOENT:
-            logger.error('Error removing %s: %s', filename, str(e))
+    warnings.warn(
+        "Use 'ipapython.ipautil.remove_file'",
+        DeprecationWarning,
+        stacklevel=2
+    )
+    return ipautil.remove_file(filename)
 
 
 def rmtree(path):
     """
     Remove a directory structure and log any exceptions raised.
     """
-    try:
-        if os.path.exists(path):
-            shutil.rmtree(path)
-    except Exception as e:
-        logger.error('Error removing %s: %s', path, str(e))
+    warnings.warn(
+        "Use 'ipapython.ipautil.rmtree'",
+        DeprecationWarning,
+        stacklevel=2
+    )
+    return ipautil.rmtree(path)
 
 
 def is_ipa_configured():
     """
-    Using the state and index install files determine if IPA is already
-    configured.
+    Use the state to determine if IPA has been configured.
     """
-    installed = False
-
-    sstore = sysrestore.StateFile(paths.SYSRESTORE)
-    fstore = sysrestore.FileStore(paths.SYSRESTORE)
-
-    for module in IPA_MODULES:
-        if sstore.has_state(module):
-            logger.debug('%s is configured', module)
-            installed = True
-        else:
-            logger.debug('%s is not configured', module)
-
-    if fstore.has_files():
-        logger.debug('filestore has files')
-        installed = True
-    else:
-        logger.debug('filestore is tracking no files')
-
-    return installed
+    warnings.warn(
+        "Use 'ipalib.facts.is_ipa_configured'",
+        DeprecationWarning,
+        stacklevel=2
+    )
+    return facts.is_ipa_configured()
 
 
 def run_script(main_function, operation_name, log_file_name=None,
@@ -900,9 +900,12 @@ def load_pkcs12(cert_files, key_password, key_nickname, ca_cert_files,
                 "The full certificate chain is not present in %s" %
                 (", ".join(cert_files)))
 
-        for nickname in trust_chain[1:]:
+        # verify CA validity and pathlen. The trust_chain list is in reverse
+        # order. trust_chain[1] is the first intermediate CA cert and must
+        # have pathlen >= 0.
+        for minpathlen, nickname in enumerate(trust_chain[1:], start=0):
             try:
-                nssdb.verify_ca_cert_validity(nickname)
+                nssdb.verify_ca_cert_validity(nickname, minpathlen)
             except ValueError as e:
                 raise ScriptError(
                     "CA certificate %s in %s is not valid: %s" %
@@ -994,6 +997,87 @@ def check_entropy():
         logger.debug("Invalid value in %s %s", paths.ENTROPY_AVAIL, e)
 
 
+def is_hidepid():
+    """Determine if /proc is mounted with hidepid=1/2 option"""
+    try:
+        os.lstat('/proc/1/stat')
+    except (FileNotFoundError, PermissionError):
+        return True
+    return False
+
+
+def in_container():
+    """Determine if we're running in a container.
+
+       virt-what will return the underlying machine information so
+       isn't usable here.
+
+       systemd-detect-virt requires the whole systemd subsystem which
+       isn't a reasonable require in a container.
+    """
+    if not is_hidepid():
+        with open('/proc/1/sched', 'r') as sched:
+            data_sched = sched.readline()
+    else:
+        data_sched = []
+
+    with open('/proc/self/cgroup', 'r') as cgroup:
+        data_cgroup = cgroup.readline()
+
+    checks = [
+        data_sched.split()[0] not in ('systemd', 'init',),
+        data_cgroup.split()[0] in ('libpod'),
+        os.path.exists('/.dockerenv'),
+        os.path.exists('/.dockerinit'),
+        os.getenv('container', None) is not None
+    ]
+
+    return any(checks)
+
+
+def check_available_memory(ca=False):
+    """
+    Raise an exception if there isn't enough memory for IPA to install.
+
+    In a container then psutil will most likely return the host memory
+    and not the container. If in a container use the cgroup values which
+    also may not be constrained but it's the best approximation.
+
+    2GB is the rule-of-thumb minimum but the server is installable with
+    much less.
+
+    The CA uses ~150MB in a fresh install.
+
+    Use Kb instead of KiB to leave a bit of slush for the OS
+    """
+    minimum_suggested = 1000 * 1000 * 1000 * 1.2
+    if not ca:
+        minimum_suggested -= 150 * 1000 * 1000
+    if in_container():
+        if os.path.exists(
+            '/sys/fs/cgroup/memory/memory.limit_in_bytes'
+        ) and os.path.exists('/sys/fs/cgroup/memory/memory.usage_in_bytes'):
+            with open('/sys/fs/cgroup/memory/memory.limit_in_bytes') as fd:
+                limit = int(fd.readline())
+            with open('/sys/fs/cgroup/memory/memory.usage_in_bytes') as fd:
+                used = int(fd.readline())
+            available = limit - used
+        else:
+            raise ScriptError(
+                "Unable to determine the amount of available RAM"
+            )
+    else:
+        # delay import of psutil. On import it opens files in /proc and
+        # can trigger a SELinux violation.
+        import psutil
+        available = psutil.virtual_memory().available
+    logger.debug("Available memory is %sB", available)
+    if available < minimum_suggested:
+        raise ScriptError(
+            "Less than the minimum 1.2GB of RAM is available, "
+            "%.2fGB available" % (available / (1024 * 1024 * 1024))
+        )
+
 def load_external_cert(files, ca_subject):
     """
     Load and verify external CA certificate chain from multiple files.
@@ -1045,9 +1129,12 @@ def load_external_cert(files, ca_subject):
                 "missing certificate with subject '%s'" %
                 (", ".join(files), issuer))
 
-        for nickname in trust_chain:
+        # verify CA validity and pathlen. The trust_chain list is in reverse
+        # order. The first entry is the signed IPA-CA and must have a
+        # pathlen of >= 0.
+        for minpathlen, nickname in enumerate(trust_chain, start=0):
             try:
-                nssdb.verify_ca_cert_validity(nickname)
+                nssdb.verify_ca_cert_validity(nickname, minpathlen)
             except ValueError as e:
                 cert, subject, issuer = cache[nickname]
                 raise ScriptError(
@@ -1069,13 +1156,26 @@ def load_external_cert(files, ca_subject):
     return cert_file, ca_file
 
 
+def get_current_platform():
+    """Get current platform (without container suffix)
+
+    'fedora' and 'fedora_container' are considered the same platform. This
+    normalization ensures that older freeipa-container images can be upgraded
+    without a platform mismatch.
+    """
+    platform = ipaplatform.NAME
+    if platform.endswith('_container'):
+        platform = platform[:-10]
+    return platform
+
+
 def store_version():
     """Store current data version and platform. This is required for check if
     upgrade is required.
     """
     sysupgrade.set_upgrade_state('ipa', 'data_version',
                                  version.VENDOR_VERSION)
-    sysupgrade.set_upgrade_state('ipa', 'platform', ipaplatform.NAME)
+    sysupgrade.set_upgrade_state('ipa', 'platform', get_current_platform())
 
 
 def check_version():
@@ -1085,12 +1185,14 @@ def check_version():
     :raise UpgradeDataNewerVersionError: older version of IPA was detected than data
     :raise UpgradeMissingVersionError: if platform or version is missing
     """
-    platform = sysupgrade.get_upgrade_state('ipa', 'platform')
-    if platform is not None:
-        if platform != ipaplatform.NAME:
+    state_platform = sysupgrade.get_upgrade_state('ipa', 'platform')
+    current_platform = get_current_platform()
+    if state_platform is not None:
+        if state_platform != current_platform:
             raise UpgradePlatformError(
                 "platform mismatch (expected '%s', current '%s')" % (
-                platform, ipaplatform.NAME)
+                    state_platform, current_platform
+                )
             )
     else:
         raise UpgradeMissingVersionError("no platform stored")
@@ -1112,14 +1214,23 @@ def check_version():
     else:
         raise UpgradeMissingVersionError("no data_version stored")
 
+
 def realm_to_serverid(realm_name):
-    return "-".join(realm_name.split("."))
+    warnings.warn(
+        "Use 'ipapython.ipaldap.realm_to_serverid'",
+        DeprecationWarning,
+        stacklevel=2
+    )
+    return ipaldap.realm_to_serverid(realm_name)
 
 
 def realm_to_ldapi_uri(realm_name):
-    serverid = realm_to_serverid(realm_name)
-    socketname = paths.SLAPD_INSTANCE_SOCKET_TEMPLATE % (serverid,)
-    return 'ldapi://' + ldapurl.ldapUrlEscape(socketname)
+    warnings.warn(
+        "Use 'ipapython.ipaldap.realm_to_ldapi_uri'",
+        DeprecationWarning,
+        stacklevel=2
+    )
+    return ipaldap.realm_to_ldapi_uri(realm_name)
 
 
 def check_creds(options, realm_name):
@@ -1340,14 +1451,12 @@ def remove_keytab(keytab_path):
 
     :param keytab_path: path to the keytab file
     """
-    try:
-        logger.debug("Removing service keytab: %s", keytab_path)
-        os.remove(keytab_path)
-    except OSError as e:
-        if e.errno != errno.ENOENT:
-            logger.warning("Failed to remove Kerberos keytab '%s': %s",
-                           keytab_path, e)
-            logger.warning("You may have to remove it manually")
+    warnings.warn(
+        "Use 'ipapython.ipautil.remove_keytab'",
+        DeprecationWarning,
+        stacklevel=2
+    )
+    return ipautil.remove_keytab(keytab_path)
 
 
 def remove_ccache(ccache_path=None, run_as=None):
@@ -1357,17 +1466,12 @@ def remove_ccache(ccache_path=None, run_as=None):
     :param ccache_path: path to the ccache file
     :param run_as: run kdestroy as this user
     """
-    logger.debug("Removing service credentials cache")
-    kdestroy_cmd = [paths.KDESTROY]
-    if ccache_path is not None:
-        logger.debug("Ccache path: '%s'", ccache_path)
-        kdestroy_cmd.extend(['-c', ccache_path])
-
-    try:
-        ipautil.run(kdestroy_cmd, runas=run_as, env={})
-    except ipautil.CalledProcessError as e:
-        logger.warning(
-            "Failed to clear Kerberos credentials cache: %s", e)
+    warnings.warn(
+        "Use 'ipapython.ipautil.remove_ccache'",
+        DeprecationWarning,
+        stacklevel=2
+    )
+    return ipautil.remove_ccache(ccache_path=ccache_path, run_as=run_as)
 
 
 def restart_dirsrv(instance_name="", capture_output=True):
@@ -1387,3 +1491,14 @@ def default_subject_base(realm_name):
 
 def default_ca_subject_dn(subject_base):
     return DN(('CN', 'Certificate Authority'), subject_base)
+
+
+def validate_mask():
+    try:
+        mask = os.umask(0)
+    finally:
+        os.umask(mask)
+    mask_str = None
+    if mask & 0b111101101 > 0:
+        mask_str = "{:04o}".format(mask)
+    return mask_str
