@@ -13,26 +13,29 @@ import glob
 import shutil
 import pwd
 import fileinput
+import ssl
 import stat
 import sys
 import tempfile
 from contextlib import contextmanager
 from augeas import Augeas
-import dns.exception
 
 from ipalib import api, x509
-from ipalib.constants import RENEWAL_CA_NAME, RA_AGENT_PROFILE
-from ipalib.install import certmonger, sysrestore
+from ipalib.constants import RENEWAL_CA_NAME, RA_AGENT_PROFILE, IPA_CA_RECORD
+from ipalib.install import certmonger
+from ipalib import sysrestore
+from ipalib.facts import is_ipa_configured
 import SSSDConfig
 import ipalib.util
 import ipalib.errors
 from ipaclient.install import timeconf
 from ipaclient.install.client import sssd_enable_ifp
+from ipalib.install.dnsforwarders import detect_resolve1_resolv_conf
 from ipaplatform import services
 from ipaplatform.tasks import tasks
 from ipapython import ipautil, version
 from ipapython import ipaldap
-from ipapython import dnsutil, directivesetter
+from ipapython import directivesetter
 from ipapython.dn import DN
 from ipaplatform.constants import constants
 from ipaplatform.paths import paths
@@ -351,8 +354,8 @@ def upgrade_adtrust_config():
                     "max smbd processes", "1000"]
             try:
                 ipautil.run(args)
-            except ipautil.CalledProcessError as e:
-                logger.warning("Error updating Samba registry: %s", e)
+            except ipautil.CalledProcessError as e2:
+                logger.warning("Error updating Samba registry: %s", e2)
         else:
             logger.warning("Error updating Samba registry: %s", e)
 
@@ -506,526 +509,6 @@ def ca_initialize_hsm_state(ca):
         ca.set_hsm_state(config)
 
 
-def named_remove_deprecated_options():
-    """
-    From IPA 3.3, persistent search is a default mechanism for new DNS zone
-    detection.
-
-    Remove psearch, zone_refresh cache_ttl and serial_autoincrement options,
-    as they have been deprecated in bind-dyndb-ldap configuration file.
-
-    When some change in named.conf is done, this functions returns True.
-    """
-
-    logger.info('[Removing deprecated DNS configuration options]')
-
-    if not bindinstance.named_conf_exists():
-        # DNS service may not be configured
-        logger.info('DNS is not configured')
-        return False
-
-    deprecated_options = ['zone_refresh', 'psearch', 'cache_ttl',
-                          'serial_autoincrement']
-    removed_options = []
-
-    try:
-        # Remove all the deprecated options
-        for option in deprecated_options:
-            value = bindinstance.named_conf_get_directive(option)
-
-            if value is not None:
-                bindinstance.named_conf_set_directive(option, None)
-                removed_options.append(option)
-
-    except IOError as e:
-        logger.error('Cannot modify DNS configuration in %s: %s',
-                     paths.NAMED_CONF, e)
-
-    # Log only the changed options
-    if not removed_options:
-        logger.debug('No changes made')
-        return False
-
-    logger.debug('The following configuration options have been removed: %s',
-                 ', '.join(removed_options))
-    return True
-
-
-def named_add_ipa_ext_conf_include():
-    """
-    Ensures named.conf does include the ipa-ext.conf file
-    """
-    if not bindinstance.named_conf_exists():
-        logger.info('DNS is not configured.')
-        return False
-
-    if not bindinstance.named_conf_include_exists(paths.NAMED_CUSTOM_CONFIG):
-        bindinstance.named_conf_add_include(paths.NAMED_CUSTOM_CONFIG)
-        return True
-    return False
-
-
-def named_add_ipa_ext_conf_file():
-    """
-    Wrapper around bindinstance.named_add_ext_conf_file().
-    Ensures named is configured before pushing the file.
-    """
-    if not bindinstance.named_conf_exists():
-        logger.info('DNS is not configured.')
-        return False
-
-    return bindinstance.named_add_ext_conf_file(
-        paths.NAMED_CUSTOM_CFG_SRC,
-        paths.NAMED_CUSTOM_CONFIG)
-
-
-def named_set_minimum_connections():
-    """
-    Sets the minimal number of connections.
-
-    When some change in named.conf is done, this functions returns True.
-    """
-
-    changed = False
-
-    logger.info('[Ensuring minimal number of connections]')
-
-    if not bindinstance.named_conf_exists():
-        # DNS service may not be configured
-        logger.info('DNS is not configured')
-        return changed
-
-    # make sure number of connections is right
-    minimum_connections = 4
-
-    try:
-        connections = bindinstance.named_conf_get_directive('connections')
-    except IOError as e:
-        logger.debug('Cannot retrieve connections option from %s: %s',
-                     paths.NAMED_CONF, e)
-        return changed
-
-    try:
-        if connections is not None:
-            connections = int(connections)
-    except ValueError:
-        # this should not happend, but there is some bad value in
-        # "connections" option, bail out
-        pass
-    else:
-        if connections is not None and connections < minimum_connections:
-            try:
-                bindinstance.named_conf_set_directive('connections',
-                                                       minimum_connections)
-                logger.debug('Connections set to %d', minimum_connections)
-            except IOError as e:
-                logger.error('Cannot update connections in %s: %s',
-                             paths.NAMED_CONF, e)
-            else:
-                changed = True
-
-    if not changed:
-        logger.debug('No changes made')
-
-    return changed
-
-
-def named_update_gssapi_configuration():
-    """
-    Update GSSAPI configuration in named.conf to a recent API.
-    tkey-gssapi-credential and tkey-domain is replaced with tkey-gssapi-keytab.
-    Details can be found in https://fedorahosted.org/freeipa/ticket/3429.
-
-    When some change in named.conf is done, this functions returns True
-    """
-
-    logger.info('[Updating GSSAPI configuration in DNS]')
-
-    if not bindinstance.named_conf_exists():
-        # DNS service may not be configured
-        logger.info('DNS is not configured')
-        return False
-
-    if sysupgrade.get_upgrade_state('named.conf', 'gssapi_updated'):
-        logger.debug('Skip GSSAPI configuration check')
-        return False
-
-    try:
-        gssapi_keytab = bindinstance.named_conf_get_directive(
-            'tkey-gssapi-keytab', bindinstance.NAMED_SECTION_OPTIONS)
-    except IOError as e:
-        logger.error('Cannot retrieve tkey-gssapi-keytab option from %s: %s',
-                     paths.NAMED_CONF, e)
-        return False
-    else:
-        if gssapi_keytab:
-            logger.debug('GSSAPI configuration already updated')
-            sysupgrade.set_upgrade_state('named.conf', 'gssapi_updated', True)
-            return False
-
-    try:
-        tkey_credential = bindinstance.named_conf_get_directive('tkey-gssapi-credential',
-                bindinstance.NAMED_SECTION_OPTIONS)
-        tkey_domain = bindinstance.named_conf_get_directive('tkey-domain',
-                bindinstance.NAMED_SECTION_OPTIONS)
-    except IOError as e:
-        logger.error('Cannot retrieve tkey-gssapi-credential option from %s: '
-                     '%s',
-                     paths.NAMED_CONF, e)
-        return False
-
-    if not tkey_credential or not tkey_domain:
-        logger.error('Either tkey-gssapi-credential or tkey-domain is missing '
-                     'in %s. Skip update.', paths.NAMED_CONF)
-        return False
-
-    try:
-        bindinstance.named_conf_set_directive(
-            'tkey-gssapi-credential', None,
-            bindinstance.NAMED_SECTION_OPTIONS)
-        bindinstance.named_conf_set_directive(
-            'tkey-domain', None,
-            bindinstance.NAMED_SECTION_OPTIONS)
-        bindinstance.named_conf_set_directive(
-            'tkey-gssapi-keytab', paths.NAMED_KEYTAB,
-            bindinstance.NAMED_SECTION_OPTIONS)
-    except IOError as e:
-        logger.error('Cannot update GSSAPI configuration in %s: %s',
-                     paths.NAMED_CONF, e)
-        return False
-    else:
-        logger.debug('GSSAPI configuration updated')
-
-    sysupgrade.set_upgrade_state('named.conf', 'gssapi_updated', True)
-    return True
-
-
-def named_update_pid_file():
-    """
-    Make sure that named reads the pid file from the right file
-    """
-    logger.info('[Updating pid-file configuration in DNS]')
-
-    if not bindinstance.named_conf_exists():
-        # DNS service may not be configured
-        logger.info('DNS is not configured')
-        return False
-
-    if sysupgrade.get_upgrade_state('named.conf', 'pid-file_updated'):
-        logger.debug('Skip pid-file configuration check')
-        return False
-
-    try:
-        pid_file = bindinstance.named_conf_get_directive(
-            'pid-file', bindinstance.NAMED_SECTION_OPTIONS)
-    except IOError as e:
-        logger.error('Cannot retrieve pid-file option from %s: %s',
-                     paths.NAMED_CONF, e)
-        return False
-    else:
-        if pid_file:
-            logger.debug('pid-file configuration already updated')
-            sysupgrade.set_upgrade_state('named.conf', 'pid-file_updated', True)
-            return False
-
-    try:
-        bindinstance.named_conf_set_directive(
-            'pid-file', paths.NAMED_PID, bindinstance.NAMED_SECTION_OPTIONS)
-    except IOError as e:
-        logger.error('Cannot update pid-file configuration in %s: %s',
-                     paths.NAMED_CONF, e)
-        return False
-    else:
-        logger.debug('pid-file configuration updated')
-
-    sysupgrade.set_upgrade_state('named.conf', 'pid-file_updated', True)
-    return True
-
-def named_enable_dnssec():
-    """
-    Enable dnssec in named.conf
-    """
-    if not bindinstance.named_conf_exists():
-        # DNS service may not be configured
-        logger.info('DNS is not configured')
-        return False
-
-    if not sysupgrade.get_upgrade_state('named.conf', 'dnssec_enabled'):
-        logger.info('[Enabling "dnssec-enable" configuration in DNS]')
-        try:
-            bindinstance.named_conf_set_directive('dnssec-enable', 'yes',
-                                                  bindinstance.NAMED_SECTION_OPTIONS,
-                                                  str_val=False)
-        except IOError as e:
-            logger.error('Cannot update dnssec-enable configuration in %s: %s',
-                         paths.NAMED_CONF, e)
-            return False
-    else:
-        logger.debug('dnssec-enabled in %s', paths.NAMED_CONF)
-
-    sysupgrade.set_upgrade_state('named.conf', 'dnssec_enabled', True)
-    return True
-
-def named_validate_dnssec():
-    """
-    Disable dnssec validation in named.conf
-
-    We can't let enable it by default, there can be non-valid dns forwarders
-    which breaks DNSSEC validation
-    """
-    if not bindinstance.named_conf_exists():
-        # DNS service may not be configured
-        logger.info('DNS is not configured')
-        return False
-
-    if (not sysupgrade.get_upgrade_state('named.conf', 'dnssec_validation_upgraded')
-        and bindinstance.named_conf_get_directive(
-                'dnssec-validation', bindinstance.NAMED_SECTION_OPTIONS,
-                str_val=False) is None):
-        # dnssec-validation is not configured, disable it
-        logger.info('[Disabling "dnssec-validate" configuration in DNS]')
-        try:
-            bindinstance.named_conf_set_directive('dnssec-validation', 'no',
-                                                  bindinstance.NAMED_SECTION_OPTIONS,
-                                                  str_val=False)
-        except IOError as e:
-            logger.error('Cannot update dnssec-validate configuration in %s: '
-                         '%s',
-                         paths.NAMED_CONF, e)
-            return False
-    else:
-        logger.debug('dnssec-validate already configured in %s',
-                     paths.NAMED_CONF)
-
-    sysupgrade.set_upgrade_state(
-        'named.conf', 'dnssec_validation_upgraded', True
-    )
-    return True
-
-
-def named_bindkey_file_option():
-    """
-    Add options bindkey_file to named.conf
-    """
-    if not bindinstance.named_conf_exists():
-        # DNS service may not be configured
-        logger.info('DNS is not configured')
-        return False
-
-    if sysupgrade.get_upgrade_state('named.conf', 'bindkey-file_updated'):
-        logger.debug('Skip bindkey-file configuration check')
-        return False
-
-    try:
-        bindkey_file = bindinstance.named_conf_get_directive(
-            'bindkey-file', bindinstance.NAMED_SECTION_OPTIONS)
-    except IOError as e:
-        logger.error('Cannot retrieve bindkey-file option from %s: %s',
-                     paths.NAMED_CONF, e)
-        return False
-    else:
-        if bindkey_file:
-            logger.debug('bindkey-file configuration already updated')
-            sysupgrade.set_upgrade_state('named.conf', 'bindkey-file_updated', True)
-            return False
-
-    logger.info('[Setting "bindkeys-file" option in named.conf]')
-    try:
-        bindinstance.named_conf_set_directive(
-            'bindkeys-file', paths.NAMED_BINDKEYS_FILE,
-            section=bindinstance.NAMED_SECTION_OPTIONS
-        )
-    except IOError as e:
-        logger.error('Cannot update bindkeys-file configuration in %s: %s',
-                     paths.NAMED_CONF, e)
-        return False
-
-
-    sysupgrade.set_upgrade_state('named.conf', 'bindkey-file_updated', True)
-    return True
-
-def named_managed_keys_dir_option():
-    """
-    Add options managed_keys_directory to named.conf
-    """
-    if not bindinstance.named_conf_exists():
-        # DNS service may not be configured
-        logger.info('DNS is not configured')
-        return False
-
-    if sysupgrade.get_upgrade_state('named.conf', 'managed-keys-directory_updated'):
-        logger.debug('Skip managed-keys-directory configuration check')
-        return False
-
-    try:
-        managed_keys = bindinstance.named_conf_get_directive('managed-keys-directory',
-                bindinstance.NAMED_SECTION_OPTIONS)
-    except IOError as e:
-        logger.error('Cannot retrieve managed-keys-directory option from %s: '
-                     '%s',
-                     paths.NAMED_CONF, e)
-        return False
-    else:
-        if managed_keys:
-            logger.debug('managed_keys_directory configuration already '
-                         'updated')
-            sysupgrade.set_upgrade_state('named.conf', 'managed-keys-directory_updated', True)
-            return False
-
-    logger.info('[Setting "managed-keys-directory" option in named.conf]')
-    try:
-        bindinstance.named_conf_set_directive('managed-keys-directory',
-                                              paths.NAMED_MANAGED_KEYS_DIR,
-                                              bindinstance.NAMED_SECTION_OPTIONS)
-    except IOError as e:
-        logger.error('Cannot update managed-keys-directory configuration in '
-                     '%s: %s',
-                     paths.NAMED_CONF, e)
-        return False
-
-
-    sysupgrade.set_upgrade_state('named.conf', 'managed-keys-directory_updated', True)
-    return True
-
-def named_root_key_include():
-    """
-    Add options managed_keys_directory to named.conf
-    """
-    if not bindinstance.named_conf_exists():
-        # DNS service may not be configured
-        logger.info('DNS is not configured')
-        return False
-
-    if sysupgrade.get_upgrade_state('named.conf', 'root_key_updated'):
-        logger.debug('Skip root key configuration check')
-        return False
-
-    try:
-        root_key = bindinstance.named_conf_include_exists(paths.NAMED_ROOT_KEY)
-    except IOError as e:
-        logger.error('Cannot check root key include in %s: %s',
-                     paths.NAMED_CONF, e)
-        return False
-    else:
-        if root_key:
-            logger.debug('root keys configuration already updated')
-            sysupgrade.set_upgrade_state('named.conf', 'root_key_updated', True)
-            return False
-
-    logger.info('[Including named root key in named.conf]')
-    try:
-        bindinstance.named_conf_add_include(paths.NAMED_ROOT_KEY)
-    except IOError as e:
-        logger.error('Cannot update named root key include in %s: %s',
-                     paths.NAMED_CONF, e)
-        return False
-
-
-    sysupgrade.set_upgrade_state('named.conf', 'root_key_updated', True)
-    return True
-
-
-def named_update_global_forwarder_policy():
-    bind = bindinstance.BindInstance()
-    if not bindinstance.named_conf_exists() or not bind.is_configured():
-        # DNS service may not be configured
-        logger.info('DNS is not configured')
-        return False
-
-    logger.info('[Checking global forwarding policy in named.conf '
-                'to avoid conflicts with automatic empty zones]')
-    if sysupgrade.get_upgrade_state(
-        'named.conf', 'forward_policy_conflict_with_empty_zones_handled'
-    ):
-        # upgrade was done already
-        return False
-
-    sysupgrade.set_upgrade_state(
-        'named.conf',
-        'forward_policy_conflict_with_empty_zones_handled',
-        True
-    )
-    try:
-        if not dnsutil.has_empty_zone_addresses(api.env.host):
-            # guess: local server does not have IP addresses from private
-            # ranges so hopefully automatic empty zones are not a problem
-            return False
-    except dns.exception.DNSException as ex:
-        logger.error(
-            'Skipping update of global DNS forwarder in named.conf: '
-            'Unable to determine if local server is using an '
-            'IP address belonging to an automatic empty zone. '
-            'Consider changing forwarding policy to "only". '
-            'DNS exception: %s', ex)
-        return False
-
-    if bindinstance.named_conf_get_directive(
-            'forward',
-            section=bindinstance.NAMED_SECTION_OPTIONS,
-            str_val=False
-    ) == 'only':
-        return False
-
-    logger.info('Global forward policy in named.conf will '
-                'be changed to "only" to avoid conflicts with '
-                'automatic empty zones')
-    bindinstance.named_conf_set_directive(
-        'forward',
-        'only',
-        section=bindinstance.NAMED_SECTION_OPTIONS,
-        str_val=False
-    )
-    return True
-
-
-def named_add_server_id():
-    """
-    DNS Locations feature requires to have configured server_id in IPA section
-    of named.conf
-    :return: if named.conf has been changed
-    """
-    bind = bindinstance.BindInstance()
-    if not bindinstance.named_conf_exists() or not bind.is_configured():
-        # DNS service may not be configured
-        logger.info('DNS is not configured')
-        return False
-
-    if sysupgrade.get_upgrade_state('named.conf', 'add_server_id'):
-        # upgrade was done already
-        return False
-
-    logger.info('[Adding server_id to named.conf]')
-    bindinstance.named_conf_set_directive('server_id', api.env.host)
-    sysupgrade.set_upgrade_state('named.conf', 'add_server_id', True)
-    return True
-
-
-def named_add_crypto_policy():
-    """Add crypto policy include
-    """
-    if not bindinstance.named_conf_exists():
-        logger.info('DNS is not configured')
-        return False
-
-    if sysupgrade.get_upgrade_state('named.conf', 'add_crypto_policy'):
-        # upgrade was done already
-        return False
-    policy_file = paths.NAMED_CRYPTO_POLICY_FILE
-    if policy_file is None:
-        # no crypto policy
-        return False
-
-    if bindinstance.named_conf_include_exists(policy_file):
-        sysupgrade.set_upgrade_state('named.conf', 'add_crypto_policy', True)
-        return False
-
-    logger.info('[Adding crypto policy include to named.conf]')
-    bindinstance.named_conf_set_directive(
-        'include', policy_file, section=bindinstance.NAMED_SECTION_OPTIONS
-    )
-    sysupgrade.set_upgrade_state('named.conf', 'add_crypto_policy', True)
-    return True
-
 
 def certificate_renewal_update(ca, kra, ds, http):
     """
@@ -1081,6 +564,10 @@ def certificate_renewal_update(ca, kra, ds, http):
                 'key-file': paths.HTTPD_KEY_FILE,
                 'ca-name': 'IPA',
                 'cert-postsave-command': template % 'restart_httpd',
+                'template-hostname': [
+                    http.fqdn,
+                    f'{IPA_CA_RECORD}.{ipautil.format_netloc(api.env.domain)}',
+                ],
             }
         )
 
@@ -1139,9 +626,9 @@ def certificate_renewal_update(ca, kra, ds, http):
 
     # Ok, now we need to stop tracking, then we can start tracking them
     # again with new configuration:
-    ca.stop_tracking_certificates()
+    ca.stop_tracking_certificates(stop_certmonger=False)
     if kra.is_installed():
-        kra.stop_tracking_certificates()
+        kra.stop_tracking_certificates(stop_certmonger=False)
     ds.stop_tracking_certificates(serverid)
     http.stop_tracking_certificates()
 
@@ -1162,6 +649,58 @@ def certificate_renewal_update(ca, kra, ds, http):
     logger.info("Certmonger certificate renewal configuration updated")
     return True
 
+
+def http_certificate_ensure_ipa_ca_dnsname(http):
+    """
+    Ensure the HTTP service certificate has the ipa-ca.$DOMAIN SAN dNSName.
+
+    This subroutine should be executed *after* ``certificate_renewal_update``,
+    which adds the name to the tracking request.  It assumes that the tracking
+    request already has the ipa-ca.$DOMAIN DNS name set, and all that is needed
+    is to resubmit the request.
+
+    If HTTP certificate is issued by a third party, print manual remediation
+    steps.
+
+    """
+    logger.info('[Adding ipa-ca alias to HTTP certificate]')
+
+    expect = f'{IPA_CA_RECORD}.{ipautil.format_netloc(api.env.domain)}'
+    cert = x509.load_certificate_from_file(paths.HTTPD_CERT_FILE)
+
+    try:
+        cert.match_hostname(expect)
+    except ssl.CertificateError:
+        if certs.is_ipa_issued_cert(api, cert):
+            request_id = certmonger.get_request_id(
+                {'cert-file': paths.HTTPD_CERT_FILE})
+            if request_id is None:
+                # shouldn't happen
+                logger.error('Could not find HTTP cert tracking request.')
+            else:
+                logger.info('Resubmitting HTTP cert tracking request')
+                certmonger.resubmit_request(request_id)
+                # NOTE: due to https://pagure.io/certmonger/issue/143, the
+                # resubmitted request, if it does not immediately succeed
+                # (fairly likely during ipa-server-upgrade) and if the notAfter
+                # date of the current cert is still far off (also likely), then
+                # Certmonger will wait 7 days before trying again (unless
+                # restarted).  There is not much we can do about that here, in
+                # the middle of ipa-server-upgrade.
+        else:
+            logger.error('HTTP certificate is issued by third party.')
+            logger.error(
+                'Obtain a new certificate with the following DNS names, \n'
+                'and install via ipa-server-certinstall(1):\n'
+                ' - %s\n'
+                ' - %s',
+                http.fqdn,
+                expect,
+            )
+    else:
+        logger.info('Certificate is OK; nothing to do')
+
+
 def copy_crl_file(old_path, new_path=None):
     """
     Copy CRL to new location, update permissions and SELinux context
@@ -1267,26 +806,24 @@ def ca_enable_pkix(ca):
     return True
 
 
-def add_ca_dns_records():
+def add_ca_dns_records(bind):
     logger.info('[Add missing CA DNS records]')
 
     if sysupgrade.get_upgrade_state('dns', 'ipa_ca_records'):
         logger.info('IPA CA DNS records already processed')
-        return
+        return False
 
     ret = api.Command['dns_is_enabled']()
     if not ret['result']:
         logger.info('DNS is not configured')
         sysupgrade.set_upgrade_state('dns', 'ipa_ca_records', True)
-        return
-
-    bind = bindinstance.BindInstance()
+        return False
 
     bind.remove_ipa_ca_cnames(api.env.domain)
-
     bind.update_system_records()
 
     sysupgrade.set_upgrade_state('dns', 'ipa_ca_records', True)
+    return True
 
 
 def find_subject_base():
@@ -1393,45 +930,6 @@ def uninstall_dogtag_9(ds, http):
     http.restart()
 
 
-def mask_named_regular():
-    """Disable named, we need to run only named-pkcs11, running both named and
-    named-pkcs can cause unexpected errors"""
-    if sysupgrade.get_upgrade_state('dns', 'regular_named_masked'):
-        return False
-
-    sysupgrade.set_upgrade_state('dns', 'regular_named_masked', True)
-
-    if bindinstance.named_conf_exists():
-        logger.info('[Masking named]')
-        named = services.service('named-regular', api)
-        try:
-            named.stop()
-        except Exception as e:
-            logger.warning('Unable to stop named service (%s)', e)
-
-        try:
-            named.mask()
-        except Exception as e:
-            logger.warning('Unable to mask named service (%s)', e)
-
-        return True
-
-    return False
-
-
-def fix_dyndb_ldap_workdir_permissions():
-    """Fix dyndb-ldap working dir permissions. DNSSEC daemons requires it"""
-    if sysupgrade.get_upgrade_state('dns', 'dyndb_ipa_workdir_perm'):
-        return
-
-    if bindinstance.named_conf_exists():
-        logger.info('[Fix bind-dyndb-ldap IPA working directory]')
-        dnskeysync = dnskeysyncinstance.DNSKeySyncInstance()
-        dnskeysync.set_dyndb_ldap_workdir_permissions()
-
-    sysupgrade.set_upgrade_state('dns', 'dyndb_ipa_workdir_perm', True)
-
-
 def fix_schema_file_syntax():
     """Fix syntax errors in schema files
 
@@ -1831,6 +1329,27 @@ def add_systemd_user_hbac():
         logger.info('Created hbac rule %s with hbacsvc=%s', rule, service)
 
 
+def add_admin_root_alias():
+    """Make root principal an alias of admin
+
+    Fix for CVE-2020-10747
+    """
+    rootprinc = "root@{}".format(api.env.realm)
+    logger.info("[Add %s alias to admin account]", rootprinc)
+    try:
+        api.Command.user_add_principal("admin", rootprinc)
+    except ipalib.errors.DuplicateEntry:
+        results = api.Command.user_find(krbprincipalname=rootprinc)
+        uid = results["result"][0]["uid"][0]
+        logger.warning(
+            "WARN: '%s' alias is assigned to user '%s'!", rootprinc, uid
+        )
+    except ipalib.errors.AlreadyContainsValueError:
+        logger.info("Alias already exists")
+    else:
+        logger.info("Added '%s' alias to admin account", rootprinc)
+
+
 def fix_permissions():
     """Fix permission of public accessible files and directories
 
@@ -1859,6 +1378,81 @@ def fix_permissions():
             os.chmod(filename, mode)
 
 
+def upgrade_bind(fstore):
+    """Update BIND named DNS server instance
+    """
+    bind = bindinstance.BindInstance(fstore, api=api)
+    bind.setup_templating(
+        fqdn=api.env.host,
+        realm_name=api.env.realm,
+        domain_name=api.env.domain
+    )
+
+    # always executed
+    add_ca_dns_records(bind)
+
+    if not bindinstance.named_conf_exists():
+        logger.info("DNS service is not configured")
+        return False
+
+    # get rid of old upgrade states
+    bind_old_upgrade_states()
+
+    # only upgrade with drop-in is missing and /etc/resolv.conf is a link to
+    # resolve1's stub resolver config file.
+    has_resolved_ipa_conf = os.path.isfile(paths.SYSTEMD_RESOLVED_IPA_CONF)
+    if not has_resolved_ipa_conf and detect_resolve1_resolv_conf():
+        bind.setup_resolv_conf()
+        logger.info("Updated systemd-resolved configuration")
+
+    if bind.is_configured() and not bind.is_running():
+        # some upgrade steps may require bind running
+        bind_started = True
+        bind.start()
+    else:
+        bind_started = False
+
+    try:
+        changed = bind.setup_named_conf(backup=True)
+        if changed:
+            logger.info("named.conf has been modified, restarting named")
+        try:
+            if bind.is_running():
+                bind.restart()
+        except ipautil.CalledProcessError as e:
+            logger.error("Failed to restart %s: %s", bind.service_name, e)
+    finally:
+        if bind_started:
+            bind.stop()
+
+    return changed
+
+
+def bind_old_upgrade_states():
+    """Remove old upgrade states
+    """
+    named_conf_states = (
+        # old states before 4.8.7
+        "gssapi_updated",
+        "pid-file_updated",
+        "dnssec-enabled_remove",
+        "bindkey-file_removed",
+        "managed-keys-directory_updated",
+        "root_key_updated",
+        "forward_policy_conflict_with_empty_zones_handled",
+        "add_server_id",
+        "add_crypto_policy",
+    )
+    dns_states = (
+        "regular_named_masked",
+        "dyndb_ipa_workdir_perm"
+    )
+    for state in named_conf_states:
+        sysupgrade.remove_upgrade_state("named.conf", state)
+    for state in dns_states:
+        sysupgrade.remove_upgrade_state("dns", state)
+
+
 def upgrade_configuration():
     """
     Execute configuration upgrade of the IPA services
@@ -1867,6 +1461,13 @@ def upgrade_configuration():
     logger.debug('IPA version %s', version.VENDOR_VERSION)
 
     fstore = sysrestore.FileStore(paths.SYSRESTORE)
+    sstore = sysrestore.StateFile(paths.SYSRESTORE)
+
+    if not sstore.has_state('installation'):
+        if is_ipa_configured():
+            sstore.backup_state('installation', 'complete', True)
+        else:
+            sstore.backup_state('installation', 'complete', False)
 
     fqdn = api.env.host
 
@@ -2078,49 +1679,7 @@ def upgrade_configuration():
     cleanup_dogtag()
     upgrade_adtrust_config()
 
-    bind = bindinstance.BindInstance(fstore)
-    if bind.is_configured() and not bind.is_running():
-        # some upgrade steps may require bind running
-        bind_started = True
-        bind.start()
-    else:
-        bind_started = False
-
-    add_ca_dns_records()
-
-    # Any of the following functions returns True iff the named.conf file
-    # has been altered
-    named_conf_changes = (
-                          named_remove_deprecated_options(),
-                          named_add_ipa_ext_conf_file(),
-                          named_add_ipa_ext_conf_include(),
-                          named_set_minimum_connections(),
-                          named_update_gssapi_configuration(),
-                          named_update_pid_file(),
-                          named_enable_dnssec(),
-                          named_validate_dnssec(),
-                          named_bindkey_file_option(),
-                          named_managed_keys_dir_option(),
-                          named_root_key_include(),
-                          named_update_global_forwarder_policy(),
-                          mask_named_regular(),
-                          fix_dyndb_ldap_workdir_permissions(),
-                          named_add_server_id(),
-                          named_add_crypto_policy(),
-                         )
-
-    if any(named_conf_changes):
-        # configuration has changed, restart the name server
-        logger.info('Changes to named.conf have been made, restart named')
-        bind = bindinstance.BindInstance(fstore)
-        try:
-            if bind.is_running():
-                bind.restart()
-        except ipautil.CalledProcessError as e:
-            logger.error("Failed to restart %s: %s", bind.service_name, e)
-
-    if bind_started:
-        bind.stop()
+    upgrade_bind(fstore)
 
     custodia = custodiainstance.CustodiaInstance(api.env.host, api.env.realm)
     custodia.upgrade_instance()
@@ -2164,6 +1723,7 @@ def upgrade_configuration():
 
     migrate_to_authselect()
     add_systemd_user_hbac()
+    add_admin_root_alias()
 
     sssd_update()
 
@@ -2191,6 +1751,10 @@ def upgrade_configuration():
     setup_pkinit(krb)
     enable_server_snippet()
 
+    # Must be executed after certificate_renewal_update
+    # (see function docstring for details)
+    http_certificate_ensure_ipa_ca_dnsname(http)
+
     if not ds_running:
         ds.stop(ds.serverid)
 
