@@ -29,11 +29,15 @@ import re
 import collections
 import itertools
 import shutil
+import copy
+import subprocess
 import tempfile
 import time
 from pipes import quote
 import configparser
 from contextlib import contextmanager
+from pkg_resources import parse_version
+import uuid
 
 import dns
 from ldif import LDIFWriter
@@ -45,6 +49,7 @@ from cryptography.hazmat.backends import default_backend
 
 from ipapython import certdb
 from ipapython import ipautil
+from ipapython.dnsutil import DNSResolver
 from ipaplatform.paths import paths
 from ipaplatform.services import knownservices
 from ipapython.dn import DN
@@ -63,67 +68,6 @@ from .firewall import Firewall
 logger = logging.getLogger(__name__)
 
 
-def setup_server_logs_collecting(host):
-    """
-    This function setup logs to be collected on host. We should collect all
-    possible logs that may be helpful to debug IPA server
-    """
-    # dirsrv logs
-    inst = host.domain.realm.replace('.', '-')
-    host.collect_log(paths.SLAPD_INSTANCE_ERROR_LOG_TEMPLATE % inst)
-    host.collect_log(paths.SLAPD_INSTANCE_ACCESS_LOG_TEMPLATE % inst)
-
-    # IPA install logs
-    host.collect_log(paths.IPASERVER_INSTALL_LOG)
-    host.collect_log(paths.IPASERVER_UNINSTALL_LOG)
-    host.collect_log(paths.IPACLIENT_INSTALL_LOG)
-    host.collect_log(paths.IPACLIENT_UNINSTALL_LOG)
-    host.collect_log(paths.IPAREPLICA_INSTALL_LOG)
-    host.collect_log(paths.IPAREPLICA_CONNCHECK_LOG)
-    host.collect_log(paths.IPAREPLICA_CA_INSTALL_LOG)
-    host.collect_log(paths.IPASERVER_KRA_INSTALL_LOG)
-    host.collect_log(paths.IPA_CUSTODIA_AUDIT_LOG)
-
-    # IPA uninstall logs
-    host.collect_log(paths.IPACLIENT_UNINSTALL_LOG)
-
-    # IPA backup and restore logs
-    host.collect_log(paths.IPARESTORE_LOG)
-    host.collect_log(paths.IPABACKUP_LOG)
-
-    # kerberos related logs
-    host.collect_log(paths.KADMIND_LOG)
-    host.collect_log(paths.KRB5KDC_LOG)
-
-    # httpd logs
-    host.collect_log(paths.VAR_LOG_HTTPD_ERROR)
-
-    # dogtag logs
-    host.collect_log(os.path.join(paths.VAR_LOG_PKI_DIR))
-
-    # selinux logs
-    host.collect_log(paths.VAR_LOG_AUDIT)
-
-    # SSSD debugging must be set after client is installed (function
-    # setup_sssd_debugging)
-
-
-def collect_logs(func):
-    def wrapper(*args):
-        try:
-            func(*args)
-        finally:
-            if hasattr(args[0], 'master'):
-                setup_server_logs_collecting(args[0].master)
-            if hasattr(args[0], 'replicas') and args[0].replicas:
-                for replica in args[0].replicas:
-                    setup_server_logs_collecting(replica)
-            if hasattr(args[0], 'clients') and args[0].clients:
-                for client in args[0].clients:
-                    setup_server_logs_collecting(client)
-    return wrapper
-
-
 def check_arguments_are(slice, instanceof):
     """
     :param: slice - tuple of integers denoting the beginning and the end
@@ -136,7 +80,8 @@ def check_arguments_are(slice, instanceof):
     def wrapper(func):
         def wrapped(*args, **kwargs):
             for i in args[slice[0]:slice[1]]:
-                assert isinstance(i, instanceof), "Wrong type: %s: %s" % (i, type(i))
+                assert isinstance(i, instanceof), "Wrong type: %s: %s" % (
+                    i, type(i))
             return func(*args, **kwargs)
         return wrapped
     return wrapper
@@ -144,9 +89,10 @@ def check_arguments_are(slice, instanceof):
 
 def prepare_reverse_zone(host, ip):
     zone = get_reverse_zone_default(ip)
-    result = host.run_command(["ipa",
-                      "dnszone-add",
-                      zone], raiseonerr=False)
+    result = host.run_command(
+        ["ipa", "dnszone-add", zone, '--skip-overlap-check'],
+        raiseonerr=False
+    )
     if result.returncode > 0:
         logger.warning("%s", result.stderr_text)
     return zone, result.returncode
@@ -158,8 +104,6 @@ def prepare_host(host):
 
         # First we try to run simple echo command to test the connection
         host.run_command(['true'], set_env=False)
-
-        host.collect_log(env_filename)
         try:
             host.transport.mkdir_recursive(host.config.test_dir)
         except IOError:
@@ -251,8 +195,10 @@ def fix_apache_semaphores(master):
         master.run_command([paths.SBIN_SERVICE, 'httpd', 'stop'],
                            raiseonerr=False)
 
-    master.run_command('for line in `ipcs -s | grep apache | cut -d " " -f 2`; '
-                       'do ipcrm -s $line; done', raiseonerr=False)
+    master.run_command(
+        'for line in `ipcs -s | grep apache ''| cut -d " " -f 2`; '
+        'do ipcrm -s $line; done', raiseonerr=False
+    )
 
 
 def unapply_fixes(host):
@@ -314,6 +260,25 @@ def enable_replication_debugging(host, log_level=0):
     ldapmodify_dm(host, logging_ldif)
 
 
+def enable_ds_audit_log(host, enabled='on'):
+    """Enable 389-ds audit log and auditfail log
+
+    :param host: the host on which audit log is configured
+    :param enabled: a string (either 'on' or 'off')
+    """
+    logger.info('Set LDAP audit log')
+    logging_ldif = textwrap.dedent("""
+        dn: cn=config
+        changetype: modify
+        replace: nsslapd-auditlog-logging-enabled
+        nsslapd-auditlog-logging-enabled: {enabled}
+        -
+        replace: nsslapd-auditfaillog-logging-enabled
+        nsslapd-auditfaillog-logging-enabled: {enabled}
+        """.format(enabled=enabled))
+    ldapmodify_dm(host, logging_ldif)
+
+
 def set_default_ttl_for_ipa_dns_zone(host, raiseonerr=True):
     args = [
         'ipa', 'dnszone-mod', host.domain.name,
@@ -332,7 +297,6 @@ def install_master(host, setup_dns=True, setup_kra=False, setup_adtrust=False,
     if domain_level is None:
         domain_level = host.config.domain_level
     check_domain_level(domain_level)
-    setup_server_logs_collecting(host)
     apply_common_fixes(host)
     fix_apache_semaphores(host)
     fw = Firewall(host)
@@ -372,6 +336,7 @@ def install_master(host, setup_dns=True, setup_kra=False, setup_adtrust=False,
     if result.returncode == 0 and not external_ca:
         # external CA step 1 doesn't have DS and KDC fully configured, yet
         enable_replication_debugging(host)
+        enable_ds_audit_log(host, 'on')
         setup_sssd_debugging(host)
         kinit_admin(host)
         if setup_dns:
@@ -443,7 +408,6 @@ def install_replica(master, replica, setup_ca=True, setup_dns=False,
         domain_level = domainlevel(master)
     check_domain_level(domain_level)
     apply_common_fixes(replica)
-    setup_server_logs_collecting(replica)
     allow_sync_ptr(master)
     fw = Firewall(replica)
     fw_services = ["freeipa-ldap", "freeipa-ldaps"]
@@ -499,6 +463,7 @@ def install_replica(master, replica, setup_ca=True, setup_dns=False,
                                  stdin_text=stdin_text)
     if result.returncode == 0:
         enable_replication_debugging(replica)
+        enable_ds_audit_log(replica, 'on')
         setup_sssd_debugging(replica)
         kinit_admin(replica)
     else:
@@ -508,8 +473,6 @@ def install_replica(master, replica, setup_ca=True, setup_dns=False,
 
 def install_client(master, client, extra_args=[], user=None,
                    password=None, unattended=True, stdin_text=None):
-    client.collect_log(paths.IPACLIENT_INSTALL_LOG)
-
     apply_common_fixes(client)
     allow_sync_ptr(master)
     # Now, for the situations where a client resides in a different subnet from
@@ -551,9 +514,6 @@ def install_adtrust(host):
     Runs ipa-adtrust-install on the client and generates SIDs for the entries.
     Configures the compat tree for the legacy clients.
     """
-
-    setup_server_logs_collecting(host)
-
     kinit_admin(host)
     host.run_command(['ipa-adtrust-install', '-U',
                       '--enable-compat',
@@ -578,11 +538,14 @@ def install_adtrust(host):
 
 
 def disable_dnssec_validation(host):
-    backup_file(host, paths.NAMED_CONF)
-    named_conf = host.get_file_contents(paths.NAMED_CONF)
+    """
+    Edits ipa-options-ext.conf snippet in order to disable dnssec validation
+    """
+    backup_file(host, paths.NAMED_CUSTOM_OPTIONS_CONF)
+    named_conf = host.get_file_contents(paths.NAMED_CUSTOM_OPTIONS_CONF)
     named_conf = re.sub(br'dnssec-validation\s*yes;', b'dnssec-validation no;',
                         named_conf)
-    host.put_file_contents(paths.NAMED_CONF, named_conf)
+    host.put_file_contents(paths.NAMED_CUSTOM_OPTIONS_CONF, named_conf)
     restart_named(host)
 
 
@@ -775,8 +738,6 @@ def setup_sssd_debugging(host):
                       paths.SSSD_CONF],
                      raiseonerr=False)
 
-    host.collect_log(os.path.join(paths.VAR_LOG_SSSD_DIR))
-
     # Clear the cache and restart SSSD
     clear_sssd_cache(host)
 
@@ -981,7 +942,6 @@ def kinit_admin(host, raiseonerr=True):
 
 def uninstall_master(host, ignore_topology_disconnect=True,
                      ignore_last_of_role=True, clean=True, verbose=False):
-    host.collect_log(paths.IPASERVER_UNINSTALL_LOG)
     uninstall_cmd = ['ipa-server-install', '--uninstall', '-U']
 
     host_domain_level = domainlevel(host)
@@ -1021,8 +981,6 @@ def uninstall_master(host, ignore_topology_disconnect=True,
 
 
 def uninstall_client(host):
-    host.collect_log(paths.IPACLIENT_UNINSTALL_LOG)
-
     host.run_command(['ipa-client-install', '--uninstall', '-U'],
                      raiseonerr=False)
     unapply_fixes(host)
@@ -1051,10 +1009,13 @@ def create_segment(master, leftnode, rightnode, suffix=DOMAIN_SUFFIX_NAME):
     lefthost = leftnode.hostname
     righthost = rightnode.hostname
     segment_name = "%s-to-%s" % (lefthost, righthost)
-    result = master.run_command(["ipa", "topologysegment-add", suffix,
-                                 segment_name,
-                                 "--leftnode=%s" % lefthost,
-                                 "--rightnode=%s" % righthost], raiseonerr=False)
+    result = master.run_command(
+        ["ipa", "topologysegment-add", suffix,
+         segment_name,
+         "--leftnode=%s" % lefthost,
+         "--rightnode=%s" % righthost],
+        raiseonerr=False
+    )
     if result.returncode == 0:
         return {'leftnode': lefthost,
                 'rightnode': righthost,
@@ -1099,6 +1060,8 @@ def _topo(name):
         topologies[name] = func
         return func
     return add_topo
+
+
 topologies = collections.OrderedDict()
 
 
@@ -1474,7 +1437,7 @@ def resolve_record(nameserver, query, rtype="SOA", retry=True, timeout=100):
     :timeout: max period of time while method will try to resolve query
      (requires retry=True)
     """
-    res = dns.resolver.Resolver()
+    res = DNSResolver()
     res.nameservers = [nameserver]
     res.lifetime = 10  # wait max 10 seconds for reply
 
@@ -1482,7 +1445,7 @@ def resolve_record(nameserver, query, rtype="SOA", retry=True, timeout=100):
 
     while time.time() < wait_until:
         try:
-            ans = res.query(query, rtype)
+            ans = res.resolve(query, rtype)
             return ans
         except dns.exception.DNSException:
             if not retry:
@@ -1490,11 +1453,58 @@ def resolve_record(nameserver, query, rtype="SOA", retry=True, timeout=100):
         time.sleep(1)
 
 
-def ipa_backup(master):
-    result = master.run_command(["ipa-backup"])
-    path_re = re.compile("^Backed up to (?P<backup>.*)$", re.MULTILINE)
-    matched = path_re.search(result.stdout_text + result.stderr_text)
-    return matched.group("backup")
+def ipa_backup(host, disable_role_check=False, raiseonerr=True):
+    """Run backup on host and return the run_command result.
+    """
+    cmd = ['ipa-backup', '-v']
+    if disable_role_check:
+        cmd.append('--disable-role-check')
+    result = host.run_command(cmd, raiseonerr=raiseonerr)
+
+    # Test for ticket 7632: check that services are restarted
+    # before the backup is compressed
+    pattern = r'.*{}.*Starting IPA service.*'.format(paths.GZIP)
+    if (re.match(pattern, result.stderr_text, re.DOTALL)):
+        raise AssertionError('IPA Services are started after compression')
+
+    return result
+
+
+def ipa_epn(
+    host, dry_run=False, from_nbdays=None, to_nbdays=None, raiseonerr=True,
+    mailtest=False,
+):
+    """Run EPN on host and return the run_command result.
+    """
+    cmd = ["ipa-epn"]
+    if dry_run:
+        cmd.append("--dry-run")
+    if mailtest:
+        cmd.append("--mail-test")
+    if from_nbdays is not None:
+        cmd.extend(("--from-nbdays", str(from_nbdays)))
+    if to_nbdays is not None:
+        cmd.extend(("--to-nbdays", str(to_nbdays)))
+    return host.run_command(cmd, raiseonerr=raiseonerr)
+
+
+def get_backup_dir(host, raiseonerr=True):
+    """Wrapper around ipa_backup: returns the backup directory.
+    """
+    result = ipa_backup(host, raiseonerr)
+
+    # Get the backup location from the command's output
+    for line in result.stderr_text.splitlines():
+        prefix = 'ipaserver.install.ipa_backup: INFO: Backed up to '
+        if line.startswith(prefix):
+            backup_path = line[len(prefix):].strip()
+            logger.info('Backup path for %s is %s', host.hostname, backup_path)
+            return backup_path
+    else:
+        if raiseonerr:
+            raise AssertionError('Backup directory not found in output')
+        else:
+            return None
 
 
 def ipa_restore(master, backup_path):
@@ -1503,15 +1513,13 @@ def ipa_restore(master, backup_path):
                         backup_path])
 
 
-def install_kra(host, domain_level=None, first_instance=False, raiseonerr=True):
+def install_kra(host, domain_level=None,
+                first_instance=False, raiseonerr=True):
     if domain_level is None:
         domain_level = domainlevel(host)
     check_domain_level(domain_level)
     command = ["ipa-kra-install", "-U", "-p", host.config.dirman_password]
-    try:
-        result = host.run_command(command, raiseonerr=raiseonerr)
-    finally:
-        setup_server_logs_collecting(host)
+    result = host.run_command(command, raiseonerr=raiseonerr)
     return result
 
 
@@ -1534,10 +1542,7 @@ def install_ca(
     if cert_files:
         for fname in cert_files:
             command.extend(['--external-cert-file', fname])
-    try:
-        result = host.run_command(command, raiseonerr=raiseonerr)
-    finally:
-        setup_server_logs_collecting(host)
+    result = host.run_command(command, raiseonerr=raiseonerr)
     return result
 
 
@@ -1691,7 +1696,7 @@ def restart_named(*args):
 
 
 def run_repeatedly(host, command, assert_zero_rc=True, test=None,
-                timeout=30, **kwargs):
+                   timeout=30, **kwargs):
     """
     Runs command on host repeatedly until it's finished successfully (returns
     0 exit code and its stdout passes the test function).
@@ -1949,7 +1954,7 @@ def ldapsearch_dm(host, base, ldap_args, scope='sub', **kwargs):
 
 
 def create_temp_file(host, directory=None, create_file=True):
-    """Creates temproray file using mktemp."""
+    """Creates temporary file using mktemp."""
     cmd = ['mktemp']
     if create_file is False:
         cmd += ['--dry-run']
@@ -1959,15 +1964,22 @@ def create_temp_file(host, directory=None, create_file=True):
 
 
 def create_active_user(host, login, password, first='test', last='user',
-                       extra_args=()):
+                       extra_args=(), krb5_trace=False):
     """Create user and do login to set password"""
     temp_password = 'Secret456789'
     kinit_admin(host)
     user_add(host, login, first=first, last=last, extra_args=extra_args,
              password=temp_password)
-    host.run_command(
-        ['kinit', login],
-        stdin_text='{0}\n{1}\n{1}\n'.format(temp_password, password))
+    if krb5_trace:
+        host.run_command(
+            "KRB5_TRACE=/dev/stdout kinit %s" % login,
+            stdin_text='{0}\n{1}\n{1}\n'.format(temp_password, password)
+        )
+    else:
+        host.run_command(
+            ['kinit', login],
+            stdin_text='{0}\n{1}\n{1}\n'.format(temp_password, password)
+        )
     kdestroy_all(host)
 
 
@@ -1989,8 +2001,135 @@ def run_command_as_user(host, user, command, *args, **kwargs):
     return host.run_command(command, *args, **kwargs)
 
 
-def kinit_as_user(host, user, password):
-    host.run_command(['kinit', user], stdin_text=password + '\n')
+def kinit_as_user(host, user, password, krb5_trace=False):
+    if krb5_trace:
+        host.run_command(
+            "KRB5_TRACE=/dev/stdout kinit %s" % user,
+            stdin_text='{0}\n'.format(password)
+        )
+    else:
+        host.run_command(['kinit', user], stdin_text='{0}\n'.format(password))
+
+
+KeyEntry = collections.namedtuple('KeyEntry',
+                                  ['kvno', 'principal', 'etype', 'key'])
+
+
+class KerberosKeyCopier:
+    """Copy Kerberos keys from a keytab to a keytab on a target host
+
+    Example:
+       Copy host/master1.ipa.test principal as MASTER$ in a temporary keytab
+
+       # host - master1.ipa.test
+       copier = KerberosKeyCopier(host)
+       realm = host.domain.realm
+       principal = copier.host_princ_template.format(
+           master=host.hostname, realm=realm)
+       replacement = {principal: f'MASTER$@{realm}'}
+
+       result = host.run_command(['mktemp'])
+       tmpname = result.stdout_text.strip()
+
+       copier.copy_keys('/etc/krb5.keytab', tmpname, replacement=replacement)
+    """
+    host_princ_template = "host/{master}@{realm}"
+    valid_etypes = ['aes256-cts-hmac-sha1-96', 'aes128-cts-hmac-sha1-96']
+
+    def __init__(self, host):
+        self.host = host
+        self.realm = host.domain.realm
+
+    def extract_key_refs(self, keytab, princ=None):
+        if princ is None:
+            princ = self.host_princ_template.format(master=self.host.hostname,
+                                                    realm=self.realm)
+        result = self.host.run_command(
+            [paths.KLIST, "-eK", "-k", keytab], log_stdout=False)
+
+        keys_to_sync = []
+        for l in result.stdout_text.splitlines():
+            if (princ in l and any(e in l for e in self.valid_etypes)):
+
+                els = l.split()
+                els[-2] = els[-2].strip('()')
+                els[-1] = els[-1].strip('()')
+                keys_to_sync.append(KeyEntry._make(els))
+
+        return keys_to_sync
+
+    def copy_key(self, keytab, keyentry):
+        # keyentry.key is a hex value of the actual key
+        # prefixed with 0x, as produced by klist -K -k.
+        # However, ktutil accepts hex value without 0x, so
+        # we should strip first two characters.
+        stdin = textwrap.dedent("""\
+        rkt {keytab}
+        addent -key -p {principal} -k {kvno} -e {etype}
+        {key}
+        wkt {keytab}
+        """).format(keytab=keytab, principal=keyentry.principal,
+                    kvno=keyentry.kvno, etype=keyentry.etype,
+                    key=keyentry.key[2:])
+
+        def get_keytab_mtime():
+            """Get keytab file mtime.
+
+            Returns mtime with sub-second precision as a string with format
+            "2020-08-25 14:35:05.980503425 +0200" or None if file does not
+            exist.
+            """
+            if self.host.transport.file_exists(keytab):
+                return self.host.run_command(
+                    ['stat', '-c', '%y', keytab]).stdout_text.strip()
+            return None
+
+        mtime_before = get_keytab_mtime()
+
+        self.host.run_command([paths.KTUTIL], stdin_text=stdin,
+                              log_stdout=False)
+        if mtime_before == get_keytab_mtime():
+            raise Exception('{} did not update keytab file "{}"'.format(
+                paths.KTUTIL, keytab))
+
+    def copy_keys(self, origin, destination, principal=None, replacement=None):
+        def sync_keys(origkeys, destkeys):
+            for origkey in origkeys:
+                copied = False
+                uptodate = False
+                if origkey.principal in replacement:
+                    origkey = copy.deepcopy(origkey)
+                    origkey.principal = replacement.get(origkey.principal)
+                for destkey in destkeys:
+                    if all([destkey.principal == origkey.principal,
+                            destkey.etype == origkey.etype]):
+                        if any([destkey.key != origkey.key,
+                                destkey.kvno != origkey.kvno]):
+                            self.copy_key(destination, origkey)
+                            copied = True
+                            break
+                        uptodate = True
+                if not (copied or uptodate):
+                    self.copy_key(destination, origkey)
+
+        if not self.host.transport.file_exists(origin):
+            raise ValueError('File "{}" does not exist'.format(origin))
+        origkeys = self.extract_key_refs(origin, princ=principal)
+        if self.host.transport.file_exists(destination):
+            destkeys = self.extract_key_refs(destination)
+            if any([origkeys is None, destkeys is None]):
+                raise Exception(
+                    'Either {} or {} are missing or unreadable'.format(
+                        origin, destination))
+            sync_keys(origkeys, destkeys)
+        else:
+            for origkey in origkeys:
+                if origkey.principal in replacement:
+                    newkey = KeyEntry._make(
+                        [origkey.kvno, replacement.get(origkey.principal),
+                         origkey.etype, origkey.key])
+                    origkey = newkey
+                self.copy_key(destination, origkey)
 
 
 class FileBackup:
@@ -2079,7 +2218,6 @@ def install_packages(host, pkgs):
     :param pkgs: packages to install, provided as a list of strings
     """
     platform = get_platform(host)
-    # Only supports RHEL 8+ and Fedora for now
     if platform in ('rhel', 'fedora'):
         install_cmd = ['/usr/bin/dnf', 'install', '-y']
     elif platform in ('ubuntu'):
@@ -2089,6 +2227,31 @@ def install_packages(host, pkgs):
     host.run_command(install_cmd + pkgs)
 
 
+def download_packages(host, pkgs):
+    """Download packages on a remote host.
+    :param host: the host where the download takes place
+    :param pkgs: packages to install, provided as a list of strings
+
+    A package can't be downloaded that is already installed.
+
+    Returns the temporary directory where the packages are.
+    The caller is responsible for cleanup.
+    """
+    platform = get_platform(host)
+    tmpdir = os.path.join('/tmp', str(uuid.uuid4()))
+    # Only supports RHEL 8+ and Fedora for now
+    if platform in ('rhel', 'fedora'):
+        install_cmd = ['/usr/bin/dnf', '-y',
+                       '--downloaddir', tmpdir,
+                       '--downloadonly',
+                       'install']
+    else:
+        raise ValueError('install_packages: unknown platform %s' % platform)
+    host.run_command(['mkdir', tmpdir])
+    host.run_command(install_cmd + pkgs)
+    return tmpdir
+
+
 def uninstall_packages(host, pkgs):
     """Uninstall packages on a remote host.
     :param host: the host where the uninstallation takes place
@@ -2102,7 +2265,7 @@ def uninstall_packages(host, pkgs):
         install_cmd = ['apt-get', 'remove', '-y']
     else:
         raise ValueError('install_packages: unknown platform %s' % platform)
-    host.run_command(install_cmd + pkgs)
+    host.run_command(install_cmd + pkgs, raiseonerr=False)
 
 
 def wait_for_request(host, request_id, timeout=120):
@@ -2113,7 +2276,7 @@ def wait_for_request(host, request_id, timeout=120):
         )
 
         state = result.stdout_text.strip()
-        print("certmonger request is in state %r", state)
+        logger.info("certmonger request is in state %s", state)
         if state in ('CA_REJECTED', 'CA_UNREACHABLE', 'CA_UNCONFIGURED',
                      'NEED_GUIDANCE', 'NEED_CA', 'MONITORING'):
             break
@@ -2124,6 +2287,34 @@ def wait_for_request(host, request_id, timeout=120):
     return state
 
 
+def wait_for_certmonger_status(host, status, request_id, timeout=120):
+    """Aggressively wait for a specific certmonger status.
+
+       This checks the status every second in order to attempt to
+       catch transient states like SUBMITTED. There are no guarantees.
+
+       :param host: the host where the uninstallation takes place
+       :param status: tuple of statuses to look for
+       :param request_id: request_id of request to check status on
+       :param timeout: max time in seconds to wait for the status
+    """
+    for _i in range(0, timeout, 1):
+        result = host.run_command(
+            "getcert list -i %s | grep status: | awk '{ print $2 }'" %
+            request_id
+        )
+
+        state = result.stdout_text.strip()
+        logger.info("certmonger request is in state %s", state)
+        if state in status:
+            break
+        time.sleep(1)
+    else:
+        raise RuntimeError("request timed out")
+
+    return state
+
+
 def wait_for_sssd_domain_status_online(host, timeout=120):
     """Wait up to timeout (in seconds) for sssd domain status to become Online
 
@@ -2146,3 +2337,141 @@ def wait_for_sssd_domain_status_online(host, timeout=120):
         time.sleep(5)
     else:
         raise RuntimeError("SSSD still offline")
+
+
+def get_sssd_version(host):
+    """Get sssd version on remote host."""
+    version = host.run_command('sssd --version').stdout_text.strip()
+    return parse_version(version)
+
+
+def run_ssh_cmd(
+    from_host=None, to_host=None, username=None, cmd=None,
+    auth_method=None, password=None, private_key_path=None,
+    expect_auth_success=True, expect_auth_failure=None,
+    verbose=True, connect_timeout=2, strict_host_key_checking=False
+):
+    """Runs an ssh connection from the controller to the host.
+       - auth_method can be either "password" or "key".
+       - In the first case, set password to the user's password ; in the
+         second case, set private_key_path to the path of the private key.
+       - If expect_auth_success or expect_auth_failure, analyze the ssh
+         client's log and check whether the selected authentication method
+         worked. expect_auth_failure takes precedence over expect_auth_success.
+       - If verbose, display the ssh client verbose log.
+       - Both expect_auth_success and verbose are True by default. Debugging
+         ssh client failures is next to impossible without the associated
+         debug log.
+       Possible enhancements:
+       - select which host to run from (currently: controller only)
+    """
+
+    if from_host is not None:
+        raise NotImplementedError(
+            "from_host must be None ; running from anywhere but the "
+            "controller is not implemented yet."
+        )
+
+    if expect_auth_failure:
+        expect_auth_success = False
+
+    if to_host is None or username is None or auth_method is None:
+        raise ValueError("host, username and auth_method are mandatory")
+    if cmd is None:
+        # cmd must run properly on all supported platforms.
+        # true(1) ("do nothing, successfully") is the obvious candidate.
+        cmd = "true"
+
+    if auth_method == "password":
+        if password is None:
+            raise ValueError(
+                "password is mandatory if auth_method == password"
+            )
+        ssh_cmd = (
+            "ssh",
+            "-v",
+            "-o", "PubkeyAuthentication=no",
+            "-o", "GSSAPIAuthentication=no",
+            "-o", "ConnectTimeout={connect_timeout}".format(
+                connect_timeout=connect_timeout
+            ),
+        )
+    elif auth_method == "key":
+        if private_key_path is None:
+            raise ValueError(
+                "private_key_path is mandatory if auth_method == key"
+            )
+        ssh_cmd = (
+            "ssh",
+            "-v",
+            "-o", "BatchMode=yes",
+            "-o", "PubkeyAuthentication=yes",
+            "-o", "GSSAPIAuthentication=no",
+            "-o", "ConnectTimeout={connect_timeout}".format(
+                connect_timeout=connect_timeout
+            ),
+        )
+    else:
+        raise ValueError(
+            "auth_method must either be password or key"
+        )
+
+    ssh_cmd_1 = list(ssh_cmd)
+    if strict_host_key_checking is True:
+        ssh_cmd_1.extend(("-o", "StrictHostKeyChecking=yes"))
+    else:
+        ssh_cmd_1.extend(("-o", "StrictHostKeyChecking=no"))
+    if auth_method == "password":
+        ssh_cmd_1 = list(("sshpass", "-p", password)) + ssh_cmd_1
+    elif auth_method == "key":
+        ssh_cmd_1.extend(("-i", private_key_path))
+    ssh_cmd_1.extend(("-l", username, to_host, cmd))
+
+    try:
+        if verbose:
+            output = "OpenSSH command: {sshcmd}".format(sshcmd=ssh_cmd_1)
+            logger.info(output)
+        remote_cmd = subprocess.Popen(
+            ssh_cmd_1,
+            shell=False,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE
+        )
+        while remote_cmd.poll() is None:
+            time.sleep(0.1)
+        return_code = remote_cmd.returncode
+        stderr = os.linesep.join(
+            str(line) for line in remote_cmd.stderr.readlines()
+        )
+        stdout = os.linesep.join(
+            str(line) for line in remote_cmd.stderr.readlines()
+        )
+        if verbose:
+            print_stdout = "Standard output: {stdout}".format(stdout=stdout)
+            print_stderr = "Standard error: {stderr}".format(stderr=stderr)
+            logger.info(print_stdout)
+            logger.info(print_stderr)
+    except Exception as e:
+        pytest.fail("Unable to run ssh command.", e)
+
+    if auth_method == "password":
+        if expect_auth_success is True:
+            assert "Authentication succeeded (keyboard-interactive)" in \
+                stderr
+            # do not assert the return code:
+            # it can be >0 if the command failed.
+        elif expect_auth_failure is True:
+            # sshpass return code: 5 for failed auth
+            assert return_code == 5
+            assert "Authentication succeeded" not in stderr
+    elif auth_method == "key":
+        if expect_auth_success is True:
+            assert "Authentication succeeded (publickey)" in stderr
+            # do not assert the return code:
+            # it can be >0 if the command failed.
+        elif expect_auth_failure is True:
+            # ssh return code: 255 for failed auth
+            assert return_code == 255
+            assert "Authentication succeeded" not in stderr
+            assert "No more authentication methods to try." in stderr
+    return (return_code, stdout, stderr)
