@@ -10,16 +10,17 @@ import re
 import os
 import logging
 import random
+import shlex
 import ssl
 from itertools import chain, repeat
 import textwrap
 import time
-import paramiko
 import pytest
 from subprocess import CalledProcessError
 
 from cryptography.hazmat.backends import default_backend
 from cryptography import x509
+from datetime import datetime, timedelta
 
 from ipalib.constants import IPAAPI_USER
 
@@ -35,7 +36,8 @@ from ipatests.pytest_ipa.integration import tasks
 from ipaplatform.tasks import tasks as platform_tasks
 from ipatests.create_external_ca import ExternalCA
 from ipatests.test_ipalib.test_x509 import good_pkcs7, badcert
-from ipapython.ipautil import realm_to_suffix
+from ipapython.ipautil import realm_to_suffix, ipa_generate_password
+from ipaserver.install.installutils import realm_to_serverid
 
 logger = logging.getLogger(__name__)
 
@@ -44,6 +46,8 @@ CONFIGURED_SERVICE = u'configuredService'
 ENABLED_SERVICE = u'enabledService'
 HIDDEN_SERVICE = u'hiddenService'
 
+DIRSRV_SLEEP = 5
+
 isrgrootx1 = (
     b'-----BEGIN CERTIFICATE-----\n'
     b'MIIFazCCA1OgAwIBAgIRAIIQz7DSQONZRGPgu2OCiwAwDQYJKoZIhvcNAQELBQAw\n'
@@ -445,6 +449,106 @@ class TestIPACommand(IntegrationTest):
         except CalledProcessError:
             pytest.fail("Password change failed when it should not")
 
+    def test_huge_password(self):
+        user = 'toolonguser'
+        hostname = 'toolong.{}'.format(self.master.domain.name)
+        huge_password = ipa_generate_password(min_len=1536)
+        original_passwd = 'Secret123'
+        master = self.master
+        base_dn = str(master.domain.basedn)
+
+        # Create a user with a password that is too long
+        tasks.kinit_admin(master)
+        add_password_stdin_text = "{pwd}\n{pwd}".format(pwd=huge_password)
+        result = master.run_command(['ipa', 'user-add', user,
+                                     '--first', user,
+                                     '--last', user,
+                                     '--password'],
+                                    stdin_text=add_password_stdin_text,
+                                    raiseonerr=False)
+        assert result.returncode != 0
+
+        # Try again with a normal password
+        add_password_stdin_text = "{pwd}\n{pwd}".format(pwd=original_passwd)
+        master.run_command(['ipa', 'user-add', user,
+                            '--first', user,
+                            '--last', user,
+                            '--password'],
+                           stdin_text=add_password_stdin_text)
+
+        # kinit as that user in order to modify the pwd
+        user_kinit_stdin_text = "{old}\n%{new}\n%{new}\n".format(
+            old=original_passwd,
+            new=original_passwd)
+        master.run_command(['kinit', user], stdin_text=user_kinit_stdin_text)
+        # sleep 1 sec (krblastpwdchange and krbpasswordexpiration have at most
+        # a 1s precision)
+        time.sleep(1)
+        # perform ldapmodify on userpassword as dir mgr
+        entry_ldif = textwrap.dedent("""
+            dn: uid={user},cn=users,cn=accounts,{base_dn}
+            changetype: modify
+            replace: userpassword
+            userpassword: {new_passwd}
+        """).format(
+            user=user,
+            base_dn=base_dn,
+            new_passwd=huge_password)
+
+        result = tasks.ldapmodify_dm(master, entry_ldif, raiseonerr=False)
+        assert result.returncode != 0
+
+        # ask_password in ipa-getkeytab will complain about too long password
+        keytab_file = os.path.join(self.master.config.test_dir,
+                                   'user.keytab')
+        password_stdin_text = "{pwd}\n{pwd}".format(pwd=huge_password)
+        result = self.master.run_command(['ipa-getkeytab',
+                                          '-p', user,
+                                          '-P',
+                                          '-k', keytab_file,
+                                          '-s', self.master.hostname],
+                                         stdin_text=password_stdin_text,
+                                         raiseonerr=False)
+        assert result.returncode != 0
+        assert "clear-text password is too long" in result.stderr_text
+
+        # Create a host with a user-set OTP that is too long
+        tasks.kinit_admin(master)
+        result = master.run_command(['ipa', 'host-add', '--force',
+                                     hostname,
+                                     '--password', huge_password],
+                                    raiseonerr=False)
+        assert result.returncode != 0
+
+        # Try again with a valid password
+        result = master.run_command(['ipa', 'host-add', '--force',
+                                     hostname,
+                                     '--password', original_passwd],
+                                    raiseonerr=False)
+        assert result.returncode == 0
+
+    def test_cleartext_password_httpd_log(self):
+        """Test to check password leak in apache error log
+
+        Host enrollment with OTP used to log the password in cleartext
+        to apache error log. This test ensures that the password should
+        not be log in cleartext.
+
+        related: https://pagure.io/freeipa/issue/8017
+        """
+        hostname = 'test.{}'.format(self.master.domain.name)
+        passwd = 'Secret123'
+
+        self.master.run_command(['ipa', 'host-add', '--force',
+                                 hostname, '--password', passwd])
+
+        # remove added host i.e cleanup
+        self.master.run_command(['ipa', 'host-del', hostname])
+
+        result = self.master.run_command(['grep', hostname,
+                                          paths.VAR_LOG_HTTPD_ERROR])
+        assert passwd not in result.stdout_text
+
     def test_change_selinuxusermaporder(self):
         """
         An update file meant to ensure a more sane default was
@@ -506,12 +610,8 @@ class TestIPACommand(IntegrationTest):
         """
         Integration test for https://pagure.io/SSSD/sssd/issue/3747
         """
-        if self.master.is_fips_mode:  # pylint: disable=no-member
-            pytest.skip("paramiko is not compatible with FIPS mode")
 
         test_user = 'test-ssh'
-        external_master_hostname = \
-            self.master.external_hostname
 
         pub_keys = []
 
@@ -521,37 +621,26 @@ class TestIPACommand(IntegrationTest):
             with open(os.path.join(
                     tmpdir, 'ssh_priv_{}'.format(i)), 'w') as fp:
                 fp.write(ssh_key_pair[0])
+                fp.write(os.linesep)
 
         tasks.kinit_admin(self.master)
         self.master.run_command(['ipa', 'user-add', test_user,
                                  '--first=tester', '--last=tester'])
 
         keys_opts = ' '.join(['--ssh "{}"'.format(k) for k in pub_keys])
-        cmd = 'ipa user-mod {} {}'.format(test_user, keys_opts)
-        self.master.run_command(cmd)
+        self.master.run_command(
+            shlex.split('ipa user-mod {} {}'.format(test_user, keys_opts))
+        )
 
         # connect with first SSH key
         first_priv_key_path = os.path.join(tmpdir, 'ssh_priv_1')
         # change private key permission to comply with SS rules
         os.chmod(first_priv_key_path, 0o600)
 
-        sshcon = paramiko.SSHClient()
-        sshcon.set_missing_host_key_policy(paramiko.AutoAddPolicy())
-
-        # first connection attempt is a workaround for
-        # https://pagure.io/SSSD/sssd/issue/3669
-        try:
-            sshcon.connect(external_master_hostname, username=test_user,
-                           key_filename=first_priv_key_path, timeout=1)
-        except (paramiko.AuthenticationException, paramiko.SSHException):
-            pass
-
-        try:
-            sshcon.connect(external_master_hostname, username=test_user,
-                           key_filename=first_priv_key_path, timeout=1)
-        except (paramiko.AuthenticationException,
-                paramiko.SSHException) as e:
-            pytest.fail('Authentication using SSH key not successful', e)
+        tasks.run_ssh_cmd(
+            to_host=self.master.external_hostname, username=test_user,
+            auth_method="key", private_key_path=first_priv_key_path
+        )
 
         journal_cmd = ['journalctl', '--since=today', '-u', 'sshd']
         result = self.master.run_command(journal_cmd)
@@ -669,7 +758,7 @@ class TestIPACommand(IntegrationTest):
         result = self.master.run_command([
             paths.IPA_CACERT_MANAGE,
             'install',
-            '/var/run/cert_not_found'], raiseonerr=False)
+            '/run/cert_not_found'], raiseonerr=False)
         assert result.returncode == 1
 
         cmd = self.master.run_command(['mktemp'])
@@ -803,22 +892,21 @@ class TestIPACommand(IntegrationTest):
         3. add an ipa user
         4. ssh from controller to master using the user created in step 3
         """
-        if self.master.is_fips_mode:  # pylint: disable=no-member
-            pytest.skip("paramiko is not compatible with FIPS mode")
 
-        sssd_version = ''
-        cmd_output = self.master.run_command(['sssd', '--version'])
-        sssd_version = platform_tasks.\
-            parse_ipa_version(cmd_output.stdout_text.strip())
-        if sssd_version.version < '2.2.0':
+        cmd = self.master.run_command(['sssd', '--version'])
+        sssd_version = platform_tasks.parse_ipa_version(
+            cmd.stdout_text.strip())
+        if sssd_version < platform_tasks.parse_ipa_version('2.2.0'):
             pytest.xfail(reason="sssd 2.2.0 unavailable in F29 nightly")
 
-        username = "testuser" + str(random.randint(200000, 9999999))
         # add ldap_deref_threshold=0 to /etc/sssd/sssd.conf
         sssd_conf_backup = tasks.FileBackup(self.master, paths.SSSD_CONF)
         with tasks.remote_sssd_config(self.master) as sssd_config:
             sssd_config.edit_domain(
                 self.master.domain, 'ldap_deref_threshold', 0)
+
+        test_user = "testuser" + str(random.randint(200000, 9999999))
+        password = "Secret123"
         try:
             self.master.run_command(['systemctl', 'restart', 'sssd.service'])
 
@@ -826,23 +914,20 @@ class TestIPACommand(IntegrationTest):
             tasks.kinit_admin(self.master)
 
             # add ipa user
-            cmd = ['ipa', 'user-add',
-                   '--first', username,
-                   '--last', username,
-                   '--password', username]
-            input_passwd = 'Secret123\nSecret123\n'
-            cmd_output = self.master.run_command(cmd, stdin_text=input_passwd)
-            assert 'Added user "%s"' % username in cmd_output.stdout_text
-            input_passwd = 'Secret123\nSecret123\nSecret123\n'
-            self.master.run_command(['kinit', username],
-                                    stdin_text=input_passwd)
+            tasks.create_active_user(
+                self.master, test_user, password=password
+            )
+            tasks.kdestroy_all(self.master)
+            tasks.kinit_as_user(
+                self.master, test_user, password
+            )
+            tasks.kdestroy_all(self.master)
+
+            tasks.run_ssh_cmd(
+                to_host=self.master.external_hostname, username=test_user,
+                auth_method="password", password=password
+            )
 
-            client = paramiko.SSHClient()
-            client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
-            client.connect(self.master.hostname,
-                           username=username,
-                           password='Secret123')
-            client.close()
         finally:
             sssd_conf_backup.restore()
             self.master.run_command(['systemctl', 'restart', 'sssd.service'])
@@ -883,6 +968,9 @@ class TestIPACommand(IntegrationTest):
         assert 'First name: %s' % (modfirst) in cmd.stdout_text
         assert 'Last name: %s' % (modlast) in cmd.stdout_text
 
+    @pytest.mark.skip_if_platform(
+        "debian", reason="Crypto policy is not supported on Debian"
+    )
     def test_enabled_tls_protocols(self):
         """Check Apache has same TLS versions enabled as crypto policy
 
@@ -918,7 +1006,6 @@ class TestIPACommand(IntegrationTest):
         assert is_tls_version_enabled('tls1_2')
         assert is_tls_version_enabled('tls1_3')
 
-    @pytest.mark.skip(reason='https://pagure.io/freeipa/issue/8151')
     def test_sss_ssh_authorizedkeys(self):
         """Login via Ssh using private-key for ipa-user should work.
 
@@ -963,7 +1050,7 @@ class TestIPACommand(IntegrationTest):
             assert ssh_pub_key in result.stdout_text
             # login to the system
             self.master.run_command(
-                ['ssh', '-o', 'PasswordAuthentication=no',
+                ['ssh', '-v', '-o', 'PasswordAuthentication=no',
                  '-o', 'IdentitiesOnly=yes', '-o', 'StrictHostKeyChecking=no',
                  '-o', 'ConnectTimeout=10', '-l', user, '-i', user_key,
                  self.master.hostname, 'true'])
@@ -1034,3 +1121,200 @@ class TestIPACommand(IntegrationTest):
             raiseonerr=False
         )
         assert result.returncode == 0
+
+    def test_ipa_adtrust_install_with_locale_issue8066(self):
+        """
+        This test checks that ipa-adtrust-install command runs successfully
+        on a system with locale en_IN.UTF-8 without displaying error below
+        'IndexError: list index out of range'
+        This is a testcase for Pagure issue
+        https://pagure.io/freeipa/issue/8066
+        """
+        # Set locale to en_IN.UTF-8 in .bashrc file to avoid reboot
+        tasks.kinit_admin(self.master)
+        BASHRC_CFG = "/root/.bashrc"
+        bashrc_backup = tasks.FileBackup(self.master, BASHRC_CFG)
+        exp_msg = "en_IN.UTF-8"
+        try:
+            self.master.run_command(
+                'echo "export LC_TIME=en_IN.UTF-8" >> ' + BASHRC_CFG
+            )
+            result = self.master.run_command('echo "$LC_TIME"')
+            assert result.stdout_text.rstrip() == exp_msg
+            # Install ipa-server-adtrust and check status
+            msg1 = (
+                "Unexpected error - see /var/log/ipaserver-install.log"
+                "for details"
+            )
+            msg2 = "IndexError: list index out of range"
+            tasks.install_packages(self.master, ["*ipa-server-trust-ad"])
+            result = self.master.run_command(
+                ["ipa-adtrust-install", "-U"], raiseonerr=False
+            )
+            assert msg1 not in result.stderr_text
+            assert msg2 not in result.stderr_text
+        finally:
+            bashrc_backup.restore()
+
+    @pytest.fixture
+    def user_creation_deletion(self):
+        # create user
+        self.testuser = 'testuser'
+        tasks.create_active_user(self.master, self.testuser, 'Secret123')
+
+        yield
+
+        # cleanup
+        tasks.kinit_admin(self.master)
+        self.master.run_command(['ipa', 'user-del', self.testuser])
+
+    def test_login_wrong_password(self, user_creation_deletion):
+        """Test ipa user login with wrong password
+
+        When ipa user login to machine using wrong password, it
+        should log proper message
+
+        related: https://github.com/SSSD/sssd/issues/5139
+        """
+        # try to login with wrong password
+        sssd_version = tasks.get_sssd_version(self.master)
+        if (sssd_version < tasks.parse_version('2.3.0')):
+            pytest.xfail('Fix is part of sssd 2.3.0 and is'
+                         ' available from fedora32 onwards')
+
+        # start to look at logs a bit before "now"
+        # https://pagure.io/freeipa/issue/8432
+        since = time.strftime(
+            '%H:%M:%S', (datetime.now() - timedelta(seconds=10)).timetuple()
+        )
+
+        password = 'WrongPassword'
+
+        tasks.run_ssh_cmd(
+            to_host=self.master.external_hostname, username=self.testuser,
+            auth_method="password", password=password,
+            expect_auth_failure=True
+        )
+
+        # check if proper message logged
+        exp_msg = ("pam_sss(sshd:auth): received for user {}: 7"
+                   " (Authentication failure)".format(self.testuser))
+        result = self.master.run_command(['journalctl',
+                                          '-u', 'sshd',
+                                          '--since={}'.format(since)])
+        assert exp_msg in result.stdout_text
+
+    def get_dirsrv_id(self):
+        serverid = realm_to_serverid(self.master.domain.realm)
+        return("dirsrv@%s.service" % serverid)
+
+    def test_ipa_nis_manage_enable(self):
+        """
+        This testcase checks if ipa-nis-manage enable
+        command enables plugin on an IPA master
+        """
+        dirsrv_service = self.get_dirsrv_id()
+        console_msg = (
+            "Enabling plugin\n"
+            "This setting will not take effect until "
+            "you restart Directory Server.\n"
+            "The rpcbind service may need to be started"
+        )
+        status_msg = "Plugin is enabled"
+        tasks.kinit_admin(self.master)
+        result = self.master.run_command(
+            ["ipa-nis-manage", "enable"],
+            stdin_text=self.master.config.admin_password,
+        )
+        assert console_msg in result.stdout_text
+        # verify using backend
+        conn = self.master.ldap_connect()  # pylint: disable=no-member
+        dn = DN(('cn', 'NIS Server'), ('cn', 'plugins'), ('cn', 'config'))
+        entry = conn.get_entry(dn)  # pylint: disable=no-member
+        nispluginstring = entry.get('nsslapd-pluginEnabled')
+        assert 'on' in nispluginstring
+        # restart for changes to take effect
+        self.master.run_command(["systemctl", "restart", dirsrv_service])
+        self.master.run_command(["systemctl", "restart", "rpcbind"])
+        time.sleep(DIRSRV_SLEEP)
+        # check status msg on the console
+        result = self.master.run_command(
+            ["ipa-nis-manage", "status"],
+            stdin_text=self.master.config.admin_password,
+        )
+        assert status_msg in result.stdout_text
+
+    def test_ipa_nis_manage_disable(self):
+        """
+        This testcase checks if ipa-nis-manage disable
+        command disable plugin on an IPA Master
+        """
+        dirsrv_service = self.get_dirsrv_id()
+        msg = (
+            "This setting will not take effect "
+            "until you restart Directory Server."
+        )
+        status_msg = "Plugin is not enabled"
+        tasks.kinit_admin(self.master)
+        result = self.master.run_command(
+            ["ipa-nis-manage", "disable"],
+            stdin_text=self.master.config.admin_password,
+        )
+        assert msg in result.stdout_text
+        # verify using backend
+        conn = self.master.ldap_connect()  # pylint: disable=no-member
+        dn = DN(('cn', 'NIS Server'), ('cn', 'plugins'), ('cn', 'config'))
+        entry = conn.get_entry(dn)  # pylint: disable=no-member
+        nispluginstring = entry.get('nsslapd-pluginEnabled')
+        assert 'off' in nispluginstring
+        # restart dirsrv for changes to take effect
+        self.master.run_command(["systemctl", "restart", dirsrv_service])
+        time.sleep(DIRSRV_SLEEP)
+        # check status msg on the console
+        result = self.master.run_command(
+            ["ipa-nis-manage", "status"],
+            stdin_text=self.master.config.admin_password,
+            raiseonerr=False,
+        )
+        assert result.returncode == 4
+        assert status_msg in result.stdout_text
+
+    def test_ipa_nis_manage_enable_incorrect_password(self):
+        """
+        This testcase checks if ipa-nis-manage enable
+        command throws error on console for invalid DS admin password
+        """
+        msg = (
+            "Insufficient access: Invalid credentials "
+            "Invalid credentials\n"
+        )
+        result = self.master.run_command(
+            ["ipa-nis-manage", "enable"],
+            stdin_text='Invalid_pwd',
+            raiseonerr=False,
+        )
+        assert result.returncode == 1
+        assert msg in result.stderr_text
+
+    def test_pkispawn_log_is_present(self):
+        """
+        This testcase checks if pkispawn logged properly.
+        It is a candidate from being moved out of test_commands.
+        """
+        result = self.master.run_command(
+            ["ls", "/var/log/pki/"]
+        )
+        pkispawnlogfile = None
+        for file in result.stdout_text.splitlines():
+            if file.startswith("pki-ca-spawn"):
+                pkispawnlogfile = file
+                break
+        assert pkispawnlogfile is not None
+        pkispawnlogfile = os.path.sep.join(("/var/log/pki", pkispawnlogfile))
+        pkispawnlog = self.master.get_file_contents(
+            pkispawnlogfile, encoding='utf-8'
+        )
+        # Totally arbitrary. pkispawn debug logs tend to be > 10KiB.
+        assert len(pkispawnlog) > 1024
+        assert "DEBUG" in pkispawnlog
+        assert "INFO" in pkispawnlog
