@@ -17,9 +17,11 @@ from datetime import datetime, timedelta
 
 import pytest
 from cryptography.hazmat.primitives import hashes
+from cryptography import x509 as crypto_x509
 
 from ipalib import x509
 from ipalib.constants import DOMAIN_LEVEL_0
+from ipalib.sysrestore import SYSRESTORE_STATEFILE, SYSRESTORE_INDEXFILE
 from ipapython.dn import DN
 from ipaplatform.constants import constants
 from ipaplatform.osinfo import osinfo
@@ -237,8 +239,23 @@ class TestInstallCA(IntegrationTest):
 
     @classmethod
     def install(cls, mh):
+        cls.master.put_file_contents(
+            os.path.join(paths.IPA_CCACHES, 'foo'),
+            'somerandomstring'
+        )
+        cls.master.run_command(
+            ['mkdir', os.path.join(paths.IPA_CCACHES, 'bar')]
+        )
         tasks.install_master(cls.master, setup_dns=False)
 
+    def test_ccaches_cleanup(self):
+        """
+        The IPA ccaches directory is cleaned up on install. Verify
+        that the file we created is now gone.
+        """
+        assert os.path.exists(os.path.join(paths.IPA_CCACHES, 'foo')) is False
+        assert os.path.exists(os.path.join(paths.IPA_CCACHES, 'bar')) is False
+
     def test_replica_ca_install_with_no_host_dns(self):
         """
         Test for ipa-ca-install --no-host-dns on a replica
@@ -345,6 +362,127 @@ class TestInstallCA(IntegrationTest):
         status = tasks.wait_for_request(self.master, request_id[0], 300)
         assert status == "MONITORING"
 
+    def test_ipa_ca_crt_permissions(self):
+        """Verify that /etc/ipa/ca.cert is mode 0644 root:root"""
+        result = self.master.run_command(
+            ["/usr/bin/stat", "-c", "%U:%G:%a", paths.IPA_CA_CRT]
+        )
+        out = str(result.stdout_text.strip())
+        (owner, group, mode) = out.split(':')
+        assert mode == "644"
+        assert owner == "root"
+        assert group == "root"
+
+    def test_cert_install_with_IPA_issued_cert(self):
+        """
+        Test replacing an IPA-issued server cert
+
+        ipa-server-certinstall can replace the web and LDAP certs.
+        A slightly different code path is taken when the replacement
+        certs are issued by IPA. Exercise that path by replacing the
+        web cert with itself.
+        """
+        self.master.run_command(['cp', '-p', paths.HTTPD_CERT_FILE, '/tmp'])
+        self.master.run_command(['cp', '-p', paths.HTTPD_KEY_FILE, '/tmp'])
+
+        passwd = self.master.get_file_contents(
+            paths.HTTPD_PASSWD_FILE_FMT.format(host=self.master.hostname)
+        )
+        self.master.run_command([
+            'ipa-server-certinstall',
+            '-p', self.master.config.dirman_password,
+            '-w',
+            '--pin', passwd,
+            '/tmp/httpd.crt',
+            '/tmp/httpd.key',
+        ])
+
+    def test_is_ipa_configured(self):
+        """Verify that the old and new methods of is_ipa_installed works
+
+           If there is an installation section then it is the status.
+
+           If not then it will fall back to looking for configured
+           services and files and use that for determination.
+        """
+        def set_installation_state(host, state):
+            """
+            Update the complete value in the installation section
+            """
+            host.run_command(
+                ['python3', '-c',
+                 'from ipalib.install import sysrestore; '
+                 'from ipaplatform.paths import paths;'
+                 'sstore = sysrestore.StateFile(paths.SYSRESTORE); '
+                 'sstore.backup_state("installation", "complete", '
+                 '{state})'.format(state=state)])
+
+        def get_installation_state(host):
+            """
+            Retrieve the installation state from new install method
+            """
+            result = host.run_command(
+                ['python3', '-c',
+                 'from ipalib.install import sysrestore; '
+                 'from ipaplatform.paths import paths;'
+                 'sstore = sysrestore.StateFile(paths.SYSRESTORE); '
+                 'print(sstore.get_state("installation", "complete"))'])
+            return result.stdout_text.strip()  # a string
+
+        # This comes from freeipa.spec and is used to determine whether
+        # an upgrade is required.
+        cmd = ['python3', '-c',
+               'import sys; from ipalib import facts; sys.exit(0 '
+               'if facts.is_ipa_configured() else 1);']
+
+        # This will use the new method since this is a fresh install,
+        # verify that it is true.
+        self.master.run_command(cmd)
+        assert get_installation_state(self.master) == 'True'
+
+        # Set complete to False which should cause the command to fail
+        # This tests the state of a failed or in-process installation.
+        set_installation_state(self.master, False)
+        result = self.master.run_command(cmd, raiseonerr=False)
+        assert result.returncode == 1
+        set_installation_state(self.master, True)
+
+        # Tweak sysrestore.state to drop installation section
+        self.master.run_command(
+            ['sed','-i', r's/\[installation\]/\[badinstallation\]/',
+             os.path.join(paths.SYSRESTORE, SYSRESTORE_STATEFILE)])
+
+        # Re-run installation check and it should fall back to old method
+        # and be successful.
+        self.master.run_command(cmd)
+        assert get_installation_state(self.master) == 'None'
+
+        # Restore installation section.
+        self.master.run_command(
+            ['sed','-i', r's/\[badinstallation\]/\[installation\]/',
+             os.path.join(paths.SYSRESTORE, SYSRESTORE_STATEFILE)])
+
+        # Uninstall and confirm that the old method reports correctly
+        # on uninstalled servers. It will exercise the old method since
+        # there is no state.
+        tasks.uninstall_master(self.master)
+
+        # ensure there is no stale state
+        result = self.master.run_command(r'test -f {}'.format(
+            os.path.join(paths.SYSRESTORE, SYSRESTORE_STATEFILE)),
+            raiseonerr=False
+        )
+        assert result.returncode == 1
+        result = self.master.run_command(r'test -f {}'.format(
+            os.path.join(paths.SYSRESTORE, SYSRESTORE_INDEXFILE)),
+            raiseonerr=False
+        )
+        assert result.returncode == 1
+
+        # Now run is_ipa_configured() and it should be False
+        result = self.master.run_command(cmd, raiseonerr=False)
+        assert result.returncode == 1
+
 
 class TestInstallWithCA_KRA1(InstallTestBase1):
 
@@ -606,24 +744,25 @@ class TestInstallMaster(IntegrationTest):
         other hand in case of restart it will change.
         """
         # listing all services
-        services = [
+        ipa_services_name = [
             "Directory", "krb5kdc", "kadmin", "named", "httpd", "ipa-custodia",
             "pki-tomcatd", "ipa-otpd", "ipa-dnskeysyncd"
         ]
 
         # checking the service status
         cmd = self.master.run_command(['ipactl', 'status'])
-        for service in services:
+        for service in ipa_services_name:
             assert f"{service} Service: RUNNING" in cmd.stdout_text
 
         # stopping few services
         service_stop = ["krb5kdc", "kadmin", "httpd"]
         for service in service_stop:
-            self.master.run_command(['systemctl', 'stop', service])
+            service_name = services.knownservices[service].systemd_name
+            self.master.run_command(['systemctl', 'stop', service_name])
 
         # checking service status
         service_start = [
-            svcs for svcs in services if svcs not in service_stop
+            svcs for svcs in ipa_services_name if svcs not in service_stop
         ]
         cmd = self.master.run_command(['ipactl', 'status'])
         for service in service_start:
@@ -636,7 +775,7 @@ class TestInstallMaster(IntegrationTest):
 
         # checking service status
         cmd = self.master.run_command(['ipactl', 'status'])
-        for service in services:
+        for service in ipa_services_name:
             assert f"{service} Service: RUNNING" in cmd.stdout_text
 
         # get process id of services
@@ -647,7 +786,7 @@ class TestInstallMaster(IntegrationTest):
 
         # checking service status
         cmd = self.master.run_command(['ipactl', 'status'])
-        for service in services:
+        for service in ipa_services_name:
             assert f"{service} Service: RUNNING" in cmd.stdout_text
 
         # check if pid for services are different
@@ -659,7 +798,7 @@ class TestInstallMaster(IntegrationTest):
 
         # checking service status
         cmd = self.master.run_command(['ipactl', 'status'])
-        for service in services:
+        for service in ipa_services_name:
             assert f"{service} Service: RUNNING" in cmd.stdout_text
 
         # check if pid for services are same
@@ -736,6 +875,17 @@ class TestInstallMaster(IntegrationTest):
                 assert key_size == 2048
             assert cert.signature_hash_algorithm.name == hashes.SHA256.name
 
+    def test_http_cert(self):
+        """
+        Test that HTTP certificate contains ipa-ca.$DOMAIN
+        DNS name.
+
+        """
+        data = self.master.get_file_contents(paths.HTTPD_CERT_FILE)
+        cert = x509.load_pem_x509_certificate(data)
+        name = f'ipa-ca.{self.master.domain.name}'
+        assert crypto_x509.DNSName(name) in cert.san_general_names
+
     def test_p11_kit_softhsm2(self):
         # check that p11-kit-proxy does not inject SoftHSM2
         result = self.master.run_command([
@@ -811,6 +961,50 @@ class TestInstallMaster(IntegrationTest):
             msg = "rpm -V found group issues for the following files: {}"
             assert group_warnings == [], msg.format(group_warnings)
 
+    def test_ds_disable_upgrade_hash(self):
+        # Test case for https://pagure.io/freeipa/issue/8315
+        # Disable password schema migration on LDAP bind
+        result = tasks.ldapsearch_dm(
+            self.master,
+            "cn=config",
+            ldap_args=["nsslapd-enable-upgrade-hash"],
+            scope="base"
+        )
+        assert "nsslapd-enable-upgrade-hash: off" in result.stdout_text
+
+    def test_ldbm_tuning(self):
+        # check db-locks in new cn=bdb subentry (1.4.3+)
+        result = tasks.ldapsearch_dm(
+            self.master,
+            "cn=bdb,cn=config,cn=ldbm database,cn=plugins,cn=config",
+            ["nsslapd-db-locks"],
+            scope="base"
+        )
+        assert "nsslapd-db-locks: 50000" in result.stdout_text
+
+        # no db-locks configuration in old global entry
+        result = tasks.ldapsearch_dm(
+            self.master,
+            "cn=config,cn=ldbm database,cn=plugins,cn=config",
+            ["nsslapd-db-locks"],
+            scope="base"
+        )
+        assert "nsslapd-db-locks" not in result.stdout_text
+
+    def test_admin_root_alias_CVE_2020_10747(self):
+        # Test for CVE-2020-10747 fix
+        # https://bugzilla.redhat.com/show_bug.cgi?id=1810160
+        rootprinc = "root@{}".format(self.master.domain.realm)
+        result = self.master.run_command(["ipa", "user-show", "admin"])
+        assert rootprinc in result.stdout_text
+
+        result = self.master.run_command(
+            ["ipa", "user-add", "root", "--first", "root", "--last", "root"],
+            raiseonerr=False
+        )
+        assert result.returncode != 0
+        assert 'user with name "root" already exists' in result.stderr_text
+
 
 class TestInstallMasterKRA(IntegrationTest):
 
@@ -854,7 +1048,7 @@ class TestInstallMasterDNS(IntegrationTest):
         related : https://pagure.io/freeipa/issue/8079
         """
         # check of /etc/named/ipa-ext.conf exist
-        assert self.master.transport.file_exists(paths.NAMED_CUSTOM_CONFIG)
+        assert self.master.transport.file_exists(paths.NAMED_CUSTOM_CONF)
 
         # check if /etc/named.conf does not contain 'allow-recursion { any; };'
         string_to_check = 'allow-recursion { any; };'
@@ -864,7 +1058,7 @@ class TestInstallMasterDNS(IntegrationTest):
 
         # check if ipa-backup command backups the /etc/named/ipa-ext.conf
         result = self.master.run_command(['ipa-backup', '-v'])
-        assert paths.NAMED_CUSTOM_CONFIG in result.stderr_text
+        assert paths.NAMED_CUSTOM_CONF in result.stderr_text
 
     def test_install_kra(self):
         tasks.install_kra(self.master, first_instance=True)
