@@ -349,6 +349,16 @@ class TestIpaClientAutomountFileRestore(IntegrationTest):
         cmd = self.clients[0].run_command(grep_automount_command)
         assert cmd.stdout_text.split() == after_ipa_client_install
 
+        self.verify_checksum_after_ipaclient_uninstall(
+            sha256nsswitch_cmd=sha256nsswitch_cmd,
+            orig_sha256=orig_sha256
+        )
+
+    def verify_checksum_after_ipaclient_uninstall(
+        self,
+        sha256nsswitch_cmd,
+        orig_sha256
+    ):
         tasks.uninstall_client(self.clients[0])
         cmd = self.clients[0].run_command(sha256nsswitch_cmd)
         assert cmd.stdout_text == orig_sha256
