@@ -183,9 +183,12 @@ ipa_class_members = {
         'normalizedns',
     ],
     'ipatests.test_integration.base.IntegrationTest': [
-        'domain',
+        {'domain': [
+            {'name': dir(str)},
+        ]},
         {'master': [
             {'config': [
+                {'dirman_dn': dir(str)},
                 {'dirman_password': dir(str)},
                 {'admin_password': dir(str)},
                 {'admin_name': dir(str)},
@@ -194,17 +197,20 @@ ipa_class_members = {
                 {'ad_admin_name': dir(str)},
                 {'ad_admin_password': dir(str)},
                 {'domain_level': dir(str)},
+                {'fips_mode': dir(bool)},
             ]},
             {'domain': [
+                {'basedn': dir(str)},
                 {'realm': dir(str)},
                 {'name': dir(str)},
             ]},
-            'hostname',
+            {'external_hostname': dir(str)},
+            {'hostname': dir(str)},
             'ip',
             'collect_log',
             {'run_command': [
                 {'stdout_text': dir(str)},
-                'stderr_text',
+                {'stderr_text': dir(str)},
                 'returncode',
             ]},
             {'transport': ['put_file', 'file_exists']},
@@ -215,6 +221,9 @@ ipa_class_members = {
         'replicas',
         'clients',
         'ad_domains',
+        {'ads': dir(list)},
+        {'ad_subdomains': dir(list)},
+        {'ad_treedomains': dir(list)},
     ]
 }
 
@@ -228,18 +237,6 @@ def fix_ipa_classes(cls):
 MANAGER.register_transform(scoped_nodes.ClassDef, fix_ipa_classes)
 
 
-def pytest_config_transform():
-    """pylint.config attribute
-    """
-    return AstroidBuilder(MANAGER).string_build(textwrap.dedent('''
-    from _pytest.config import get_config
-    config = get_config()
-    '''))
-
-
-register_module_extender(MANAGER, 'pytest', pytest_config_transform)
-
-
 def ipaplatform_constants_transform():
     return AstroidBuilder(MANAGER).string_build(textwrap.dedent('''
     from ipaplatform.base.constants import constants
@@ -283,6 +280,18 @@ register_module_extender(MANAGER, 'ipaplatform.tasks',
                          ipaplatform_tasks_transform)
 
 
+def ipalib_request_transform():
+    """ipalib.request.context attribute
+    """
+    return AstroidBuilder(MANAGER).string_build(textwrap.dedent('''
+    from ipalib.request import context
+    context._pylint_attr = Connection("_pylint", lambda: None)
+    '''))
+
+
+register_module_extender(MANAGER, 'ipalib.request', ipalib_request_transform)
+
+
 class IPAChecker(BaseChecker):
     __implements__ = IAstroidChecker
 
@@ -432,6 +441,7 @@ AstroidBuilder(MANAGER).string_build(textwrap.dedent(
     api.env.ca_host = ''
     api.env.ca_install_port = None
     api.env.ca_port = 0
+    api.env.certmonger_wait_timeout = 0
     api.env.conf = ''  # object
     api.env.conf_default = ''  # object
     api.env.confdir = ''  # object
@@ -441,6 +451,7 @@ AstroidBuilder(MANAGER).string_build(textwrap.dedent(
     api.env.container_automember = DN()
     api.env.container_automount = DN()
     api.env.container_ca = DN()
+    api.env.container_ca_renewal = DN()
     api.env.container_caacl = DN()
     api.env.container_certmap = DN()
     api.env.container_certmaprules = DN()
@@ -498,6 +509,7 @@ AstroidBuilder(MANAGER).string_build(textwrap.dedent(
     api.env.force_schema_check = False
     api.env.home = ''  # object
     api.env.host = ''
+    api.env.host_princ = ''
     api.env.http_timeout = 0
     api.env.in_server = False  # object
     api.env.in_tree = False  # object
@@ -523,6 +535,7 @@ AstroidBuilder(MANAGER).string_build(textwrap.dedent(
     api.env.script = ''  # object
     api.env.site_packages = ''  # object
     api.env.skip_version_check = False
+    api.env.smb_princ = ''
     api.env.startup_timeout = 0
     api.env.startup_traceback = False
     api.env.tls_ca_cert = ''  # object
@@ -535,3 +548,33 @@ AstroidBuilder(MANAGER).string_build(textwrap.dedent(
     api.env.webui_prod = True
     """
 ))
+
+# dnspython 2.x introduces enums and creates module level globals from them
+# pylint does not understand the trick
+AstroidBuilder(MANAGER).string_build(textwrap.dedent(
+    """
+    import dns.flags
+    import dns.rdataclass
+    import dns.rdatatype
+
+    dns.flags.AD = 0
+    dns.flags.CD = 0
+    dns.flags.DO = 0
+    dns.flags.RD = 0
+
+    dns.rdataclass.IN = 0
+
+    dns.rdatatype.A = 0
+    dns.rdatatype.AAAA = 0
+    dns.rdatatype.CNAME = 0
+    dns.rdatatype.DNSKEY = 0
+    dns.rdatatype.MX = 0
+    dns.rdatatype.NS = 0
+    dns.rdatatype.PTR = 0
+    dns.rdatatype.RRSIG = 0
+    dns.rdatatype.SOA = 0
+    dns.rdatatype.SRV = 0
+    dns.rdatatype.TXT = 0
+    dns.rdatatype.URI = 0
+    """
+))
